package lmntaleditor;

import java.awt.Image;
import java.awt.Toolkit;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Map;
import java.util.Properties;

public class Env {

	static public final String APP_NAME = "LMNtal Editor";
	static public final String APP_VERSION = "1.0.0";
	static public final String APP_DATE = "2009/1/19";

	static public Env env = null;
    static private final String ENV_FILE = "env.txt";

    static public final String SLIM_DIR_NAME = "slim-0.4.0";
    static public final String UNYO_DIR_NAME = "unyo0_5_1";
    static public final String LTL2BA_DIR_NAME = "ltl2ba-1.1";

    static private Properties prop = new Properties();

    public Env(){
    	env = this;
    	try {
    		InputStream in = Env.getInputStreamOfFile(ENV_FILE);
			prop.load(in);
			in.close();
        } catch (Exception e) {
        	System.err.println("read error. check "+ENV_FILE);
            System.exit(0);
        }
    }

    static public void save(){
		try {
			FileOutputStream out = new FileOutputStream(ENV_FILE);
			prop.store(out, APP_NAME+" "+APP_VERSION);
	        out.close();
		} catch (IOException e) {
			System.err.println("save error.check "+ENV_FILE);
		}
    }

    static public String get(String key){
    	return prop.getProperty(key);
    }

    static public int getInt(String key){
    	return Integer.valueOf(prop.getProperty(key));
    }

    static public boolean is(String key){
    	return Boolean.valueOf(prop.getProperty(key));
    }

    static public void setProcessEnvironment(Map<String,String> map){
    	String lmntalHome = (new File("lmntal")).getAbsolutePath();
    	if(File.separatorChar=='\\'){
    		lmntalHome = lmntalHome.replace('\\','/');
    	}
		map.put("LMNTAL_HOME",lmntalHome);

    	if(File.pathSeparatorChar==';'){
    		String bin = "";
    		bin += get("WINDOWS_CYGWIN_DIR")+File.separatorChar+"bin;";
    		bin += get("WINDOWS_CYGWIN_DIR")+File.separatorChar+"usr"+File.separatorChar+"bin;";
    		bin += get("WINDOWS_CYGWIN_DIR")+File.separatorChar+"usr"+File.separatorChar+"local"+File.separatorChar+"bin;";
    		if(map.get("path")!=null) map.put("path",bin+map.get("path"));
    		if(map.get("Path")!=null) map.put("Path",bin+map.get("Path"));
    		if(map.get("PATH")!=null) map.put("PATH",bin+map.get("PATH"));
		}
    }

    static public String getBinaryAbsolutePath(String cmd){
    	if(File.pathSeparatorChar==';'){
    		String bin = get("WINDOWS_CYGWIN_DIR")+File.separatorChar+"bin"+File.separatorChar+cmd+".exe";
    		String usrBin = get("WINDOWS_CYGWIN_DIR")+File.separatorChar+"usr"+File.separatorChar+"bin"+File.separatorChar+cmd+".exe";
    		if((new File(bin)).exists()){
    			return bin;
    		}else if((new File(usrBin)).exists()){
    			return usrBin;
    		}else{
    			return cmd;
    		}
    	}else{
    		String bin = File.separatorChar+"bin"+File.separatorChar+cmd;
    		String usrBin = File.separatorChar+"usr"+File.separatorChar+"bin"+File.separatorChar+cmd;
    		if((new File(bin)).exists()){
    			return bin;
    		}else if((new File(usrBin)).exists()){
    			return usrBin;
    		}else{
    			return cmd;
    		}
    	}
    }

    static public String getSlimBinaryName(){
    	if(File.pathSeparatorChar==';'){
    		return "slim.exe";
		}else{
			return "slim";
		}
    }

    static public String getLtl2baBinaryName(){
    	if(File.pathSeparatorChar==';'){
    		return "ltl2ba.exe";
		}else{
			return "ltl2ba";
		}
    }

    static public double getPercentage(String key,double per){
    	String str = get(key);
		if(str.matches("^[0-9]{1,3}%$")){
			int t = Integer.parseInt(str.substring(0,str.indexOf('%')));
			if(t<0){ t=0; }else if(t>100){ t=100; }
			per = t/100.0;
        }
		return per;
	}

    static public void set(String key,String value){
    	prop.setProperty(key, value);
    }

    static public void set(String key,boolean value){
    	prop.setProperty(key, String.valueOf(value));
    }

    static public void set(String key,int value){
    	prop.setProperty(key, String.valueOf(value));
    }

	static public void setPercentage(String key,double per){
		int res = (int)(per*100);
		if(res<0){
			set(key,"0%");
		}else if(res>100){
			set(key,"100%");
		}else{
			set(key,res+"%");
		}
	}

    // jarե벽ΥեϤκۼ
    static public InputStream getInputStreamOfFile(String filename){
    	InputStream in = null;
    	File file = new File(filename);
    	try {
    		if(file.exists()){
    			in = new FileInputStream(file);
    		}else{
    			in = env.getClass().getResourceAsStream("/"+filename);
    		}
    	} catch (Exception e) {
    		e.printStackTrace();
    	}
    	return in;
    }

    // jarե벽ΥեϤκۼ
    static public Image getImageOfFile(String filename){
    	File file = new File(filename);
    	if(file.exists()){
    		return Toolkit.getDefaultToolkit().getImage(file.getPath());
    	}else{
    		URL fileUrl = env.getClass().getResource("/"+filename);
    		return Toolkit.getDefaultToolkit().getImage(fileUrl);
    	}
    }

}