
package lmntaleditor.editor;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

import lmntaleditor.*;
import lmntaleditor.ltl.LtlButtonPanel;
import lmntaleditor.runner.*;

public class EditorButtonPanel extends JPanel implements ActionListener {

	private EditorPanel editorPanel;

	private LmntalRunner lmntalRunner;
	private SlimRunner slimRunner;
	private UnyoRunner unyoRunner;

	private JPanel buttonPanel;
	public JButton lmntalButton;
	public JButton lmntalgButton;
	public JButton unyoButton;
	public JButton mcprofilerButton;
	public JButton slimButton;
	public JButton sviewerButton;
	public JButton nullButton;
	public JButton killButton;

	private JPanel labelPanel;
	private JLabel rowColumnStatus;
	private JLabel fileStatus;

	private int textRow;
	private int textColumn;

	public EditorButtonPanel(EditorPanel editorPanel){
		this.editorPanel = editorPanel;

		setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));

		//ܥ
		buttonPanel = new JPanel(new GridLayout(2,4));

		lmntalButton = new JButton(Lang.m[11]);
		lmntalButton.addActionListener(this);
		buttonPanel.add(lmntalButton);

		lmntalgButton = new JButton(Lang.m[12]);
		lmntalgButton.addActionListener(this);
		buttonPanel.add(lmntalgButton);

		unyoButton = new JButton(Lang.m[13]);
		unyoButton.addActionListener(this);
		buttonPanel.add(unyoButton);

		mcprofilerButton = new JButton(Lang.m[21]);
		mcprofilerButton.addActionListener(this);
		buttonPanel.add(mcprofilerButton);

		slimButton = new JButton(Lang.m[14]);
		slimButton.addActionListener(this);
		buttonPanel.add(slimButton);

		sviewerButton = new JButton(Lang.m[15]);
		sviewerButton.addActionListener(this);
		buttonPanel.add(sviewerButton);

		/*
		sviewerlButton = new JButton("(LTL)"+Lang.m[15]);
		sviewerlButton.addActionListener(this);
		buttonPanel.add(sviewerlButton);
		*/
		nullButton = new JButton();
		buttonPanel.add(nullButton);

		killButton = new JButton(Lang.m[20]);
		killButton.addActionListener(this);
		buttonPanel.add(killButton);

		setAllEnable(true);
		add(buttonPanel);

		//٥
		labelPanel = new JPanel(new GridLayout(1,2));

		fileStatus = new JLabel();
		updateFileStatus();
		labelPanel.add(fileStatus);

		rowColumnStatus = new JLabel();
		rowColumnStatus.setHorizontalAlignment(JLabel.RIGHT);
		setRowColumn(1,1);
		labelPanel.add(rowColumnStatus);

		add(labelPanel);

	}

	public void updateFileStatus(){
		String fileMark = "";
		if(editorPanel.isChanged()){
			fileMark = "["+Lang.f[5]+"]";
		}
		fileStatus.setText(" "+fileMark+" "+editorPanel.getFileName());
	}

	public void setRowColumn(int r, int c){
		this.textRow=r;
		this.textColumn=c;
		String rowColumn = textRow + ":" + textColumn;
		rowColumnStatus.setText(rowColumn);
	}

	public void setAllEnable(boolean enable){
		nullButton.setEnabled(enable);

		lmntalButton.setEnabled(enable);
		lmntalgButton.setEnabled(enable);
		unyoButton.setEnabled(enable);
		slimButton.setEnabled(enable);
		sviewerButton.setEnabled(enable);
		mcprofilerButton.setEnabled(enable);

		killButton.setEnabled(!enable);
	}

	private void setButtonEnable(boolean enable){
		setAllEnable(enable);
		FrontEnd.mainFrame.toolTab.ltlPanel.ltlButtonPanel.setAllEnable(enable);
	}

	public void actionPerformed(ActionEvent e) {
		JButton src = (JButton)e.getSource();

		if (src == lmntalButton) {

			if(editorPanel.isChanged()){
				editorPanel.fileSave();
			}

			setButtonEnable(false);

			FrontEnd.mainFrame.toolTab.setTab("System");

			FrontEnd.println("(LMNtal) Doing...");
			lmntalRunner = new LmntalRunner(Env.get("LMNTAL_OPTION"));
			lmntalRunner.run();
			(new Thread(new Runnable() { public void run() {
				while(lmntalRunner.isRunning()){
					FrontEnd.sleep(200);
				}
				FrontEnd.println("(LMNtal) Done!");
				lmntalRunner = null;
				javax.swing.SwingUtilities.invokeLater(new Runnable(){public void run() {
					setButtonEnable(true);
				}});
			}})).start();

		}else if (src == lmntalgButton) {

			if(editorPanel.isChanged()){
				editorPanel.fileSave();
			}

			setButtonEnable(false);

			FrontEnd.mainFrame.toolTab.setTab("System");

			FrontEnd.println("(LMNtal) Doing...");
			lmntalRunner = new LmntalRunner("-g");
			lmntalRunner.run();
			(new Thread(new Runnable() { public void run() {
				while(lmntalRunner.isRunning()){
					FrontEnd.sleep(200);
				}
				FrontEnd.println("(LMNtal) Done!");
				lmntalRunner = null;
				javax.swing.SwingUtilities.invokeLater(new Runnable(){public void run() {
					setButtonEnable(true);
				}});
			}})).start();

		}else if (src == unyoButton) {

			if(editorPanel.isChanged()){
				editorPanel.fileSave();
			}

			setButtonEnable(false);

			FrontEnd.mainFrame.toolTab.setTab("System");

			FrontEnd.println("(UNYO) Doing...");
			unyoRunner = new UnyoRunner(Env.get("LMNTAL_OPTION"));
			unyoRunner.run();
			(new Thread(new Runnable() { public void run() {
				while(unyoRunner.isRunning()){
					FrontEnd.sleep(200);
				}
				FrontEnd.println("(UNYO) Done!");
				unyoRunner = null;
				javax.swing.SwingUtilities.invokeLater(new Runnable(){public void run() {
					setButtonEnable(true);
				}});
			}})).start();

		}else if (src == slimButton) {

			if(editorPanel.isChanged()){
				editorPanel.fileSave();
			}

			setButtonEnable(false);

			FrontEnd.mainFrame.toolTab.setTab("System");

			FrontEnd.println("(SLIM) Doing...");
			slimRunner = new SlimRunner(Env.get("SLIM_OPTION"));
			slimRunner.run();
			(new Thread(new Runnable() { public void run() {
				while(slimRunner.isRunning()){
					FrontEnd.sleep(200);
				}
				FrontEnd.println("(SLIM) Done!");
				slimRunner = null;
				javax.swing.SwingUtilities.invokeLater(new Runnable(){public void run() {
					setButtonEnable(true);
				}});
			}})).start();

		}else if (src == sviewerButton) {

			if(editorPanel.isChanged()){
				editorPanel.fileSave();
			}

			setButtonEnable(false);

			FrontEnd.mainFrame.toolTab.setTab("System");

			FrontEnd.println("(StateViewer) Doing...");
			slimRunner = new SlimRunner("--nd --hideruleset");
			//slimRunner = new SlimRunner("--nd --hideruleset");
			slimRunner.setBuffering(true);
			slimRunner.run();
			(new Thread(new Runnable() { public void run() {
				while(slimRunner.isRunning()){
					FrontEnd.sleep(200);
				}
				if(slimRunner.isSuccess()){
					FrontEnd.mainFrame.toolTab.statePanel.start(slimRunner.getBufferString(),false);
				}
				slimRunner = null;
				javax.swing.SwingUtilities.invokeLater(new Runnable(){public void run() {
					setButtonEnable(true);
				}});
			}})).start();
/*
		}else if (src == sviewerlButton) {

			if(editorPanel.isChanged()){
				editorPanel.fileSave();
			}

			setButtonEnable(false);

			FrontEnd.mainFrame.toolTab.setTab("System");

			FrontEnd.println("(StateViewer) Doing...");
			slimRunner = new SlimRunner("--ltl_nd --hideruleset");
			slimRunner.setBuffering(true);
			slimRunner.run();
			(new Thread(new Runnable() { public void run() {
				while(slimRunner.isRunning()){
					FrontEnd.sleep(200);
				}
				if(slimRunner.isSuccess()){
					FrontEnd.mainFrame.toolTab.statePanel.start(slimRunner.getBufferString());
				}
				slimRunner = null;
				javax.swing.SwingUtilities.invokeLater(new Runnable(){public void run() {
					setButtonEnable(true);
				}});
			}})).start();
*/

		}else if (src == mcprofilerButton) {

			if(editorPanel.isChanged()){
				editorPanel.fileSave();
			}

			setButtonEnable(false);

			FrontEnd.mainFrame.toolTab.setTab("MCProfiler");

			FrontEnd.println("(MCProfiler) Doing...");
			slimRunner = new SlimRunner("--nd_dump --hideruleset");
			slimRunner.setOutputGetter(FrontEnd.mainFrame.toolTab.mcprofilePanel);

			slimRunner.run();
			(new Thread(new Runnable() { public void run() {
				while(slimRunner.isRunning()){
					FrontEnd.sleep(200);
				}
				FrontEnd.println("(MCProfiler) Done!");
				slimRunner = null;
				javax.swing.SwingUtilities.invokeLater(new Runnable(){public void run() {
					setButtonEnable(true);
				}});
			}})).start();


		}else if (src == killButton) {

			if(lmntalRunner!=null) lmntalRunner.kill();
			if(slimRunner!=null) slimRunner.kill();
			if(unyoRunner!=null) unyoRunner.kill();
			FrontEnd.mainFrame.toolTab.ltlPanel.ltlButtonPanel.runnerKill();
			FrontEnd.errPrintln("Kill");

		}
	}

}
