package lmntaleditor.frame;

import java.awt.Frame;
import java.awt.Point;
import java.awt.Window;
import java.util.HashSet;

import javax.swing.JFrame;
import javax.swing.JSplitPane;
import javax.swing.WindowConstants;

import lmntaleditor.Env;
import lmntaleditor.FrontEnd;
import lmntaleditor.editor.EditorPanel;
import lmntaleditor.util.CommonFontUser;

public class MainFrame extends JFrame{

	public HashSet<Window> childFrames;

	public MainMenuBar mainMenuBar;
	public EditorPanel editorPanel;
	public JSplitPane jsp;
	public ToolTab toolTab;

	public MainFrame(){

		childFrames = new HashSet<Window>();

		setSize(Env.getInt("WINDOW_WIDTH"),Env.getInt("WINDOW_HEIGHT"));
        setLocation(new Point(Env.getInt("WINDOW_X"),Env.getInt("WINDOW_Y")));
        setTitle(Env.APP_NAME);
        setIconImage(Env.getImageOfFile("img/icon.gif"));
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

        mainMenuBar = new MainMenuBar();
		setJMenuBar(mainMenuBar);

		editorPanel = new EditorPanel();

		toolTab = new ToolTab();

		double editerPer = Env.getPercentage("WINDOW_EDITER_PERCENTAGE",50);
		jsp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, editorPanel, toolTab);
		jsp.setOneTouchExpandable(true);
        jsp.setResizeWeight(0.5);
        jsp.setDividerLocation((int)(Env.getInt("WINDOW_WIDTH")*editerPer));
        setContentPane(jsp);

        addWindowListener(new MainWindowListener(this));

        setVisible(true);

	}

	public void addChildWindow(Window window){
		childFrames.add(window);
	}

	public void removeChildWindow(Window window){
		childFrames.remove(window);
	}

	public void setAllChildWindowVisible(boolean visible){
		for(Window w : childFrames){
			w.setVisible(visible);
		}
	}

	public void exit(){
		Env.set("WINDOW_X", getLocation().x);
		Env.set("WINDOW_Y", getLocation().y);
		Env.set("WINDOW_WIDTH", getWidth());
		Env.set("WINDOW_HEIGHT", getHeight());
		Env.setPercentage("WINDOW_EDITER_PERCENTAGE",(double)jsp.getDividerLocation()/(double)getWidth());
	}

}
