package lmntaleditor.ltl;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import lmntaleditor.Env;
import lmntaleditor.FrontEnd;
import lmntaleditor.editor.EditorButtonPanel;
import lmntaleditor.editor.EditorPanel;
import lmntaleditor.runner.SlimRunner;
import lmntaleditor.util.FixFlowLayout;

public class LtlButtonPanel extends JPanel implements ActionListener {

	public LtlPanel ltlPanel;

	private SlimRunner slimRunner;

	private JLabel label;
	private JComboBox combo;
	private String fileNumberList[] = {"0","1","2","3","4","5","6","7","8","9"};
	private JButton saveButton;
	private JButton loadButton;

	private JButton ltlButton;
	private JButton ltlallButton;
	private JButton ltlsvButton;

	LtlButtonPanel(LtlPanel ltlPanel){

		this.ltlPanel = ltlPanel;

		setLayout(new FixFlowLayout());

		JPanel filePanel = new JPanel();
		filePanel.setBorder(new TitledBorder("LTL File"));

		label = new JLabel("File No.");
		filePanel.add(label);
		combo = new JComboBox(fileNumberList);
		Dimension dim = combo.getPreferredSize();
		combo.setPreferredSize(new Dimension(dim.width+10, dim.height));
		filePanel.add(combo);

		loadButton = new JButton("Load");
		loadButton.addActionListener(this);
		filePanel.add(loadButton);

		saveButton = new JButton("Save");
		saveButton.addActionListener(this);
		filePanel.add(saveButton);
		add(filePanel);


		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel,BoxLayout.X_AXIS));
		buttonPanel.setBorder(new TitledBorder("LTL Model Check"));

		ltlButton = new JButton("slim --ltl");
		ltlButton.addActionListener(this);
		buttonPanel.add(ltlButton);

		ltlallButton = new JButton("slim --ltl_all");
		ltlallButton.addActionListener(this);
		buttonPanel.add(ltlallButton);

		ltlsvButton = new JButton("LTL StateViewer");
		ltlsvButton.addActionListener(this);
		buttonPanel.add(ltlsvButton);

		setAllEnable(true);

		add(buttonPanel);

	}

	public void setAllEnable(boolean enable){
		ltlButton.setEnabled(enable);
		ltlallButton.setEnabled(enable);
		ltlsvButton.setEnabled(enable);
	}

	private void setButtonEnable(boolean enable){
		setAllEnable(enable);
		FrontEnd.mainFrame.editorPanel.buttonPanel.setAllEnable(enable);
	}

	public void setSelected(String no){
		combo.setSelectedItem(no);
	}

	public String getSelectedNo(){
		return (String)combo.getSelectedItem();
	}

	public void actionPerformed(ActionEvent e) {
		Object src = e.getSource();
		if(src==loadButton){
			ltlPanel.loadFile(getSelectedNo());
		}else if(src==saveButton){
			ltlPanel.saveFile(getSelectedNo());
		}else if(src==ltlButton||src==ltlallButton){

			EditorPanel editorPanel = FrontEnd.mainFrame.editorPanel;

			if(editorPanel.isChanged()){
				editorPanel.fileSave();
			}

			ltlPanel.saveFile(getSelectedNo());

			setButtonEnable(false);

			FrontEnd.mainFrame.toolTab.setTab("System");

			FrontEnd.println("(SLIM) Doing...");

			String option = "";
			if(src==ltlButton){
				option = "--ltl";
			}else if(src==ltlallButton){
				option = "--ltl_all";
			}

			slimRunner = new SlimRunner(option);
			slimRunner.setSymbolFile(ltlPanel.getSymbolFile(getSelectedNo()));
			slimRunner.setNcFile(ltlPanel.getNcFile(getSelectedNo()));
			slimRunner.run();
			(new Thread(new Runnable() { public void run() {
				while(slimRunner.isRunning()){
					FrontEnd.sleep(200);
				}
				FrontEnd.println("(SLIM) Done!");
				slimRunner = null;
				javax.swing.SwingUtilities.invokeLater(new Runnable(){public void run() {
					setButtonEnable(true);
				}});
			}})).start();

		}else if(src==ltlsvButton){

			EditorPanel editorPanel = FrontEnd.mainFrame.editorPanel;

			if(editorPanel.isChanged()){
				editorPanel.fileSave();
			}

			ltlPanel.saveFile(getSelectedNo());

			setButtonEnable(false);

			FrontEnd.mainFrame.toolTab.setTab("System");

			FrontEnd.println("(StateViewer) Doing...");

			slimRunner = new SlimRunner("--ltl_nd");
			slimRunner.setSymbolFile(ltlPanel.getSymbolFile(getSelectedNo()));
			slimRunner.setNcFile(ltlPanel.getNcFile(getSelectedNo()));
			slimRunner.setBuffering(true);
			slimRunner.run();
			(new Thread(new Runnable() { public void run() {
				while(slimRunner.isRunning()){
					FrontEnd.sleep(200);
				}
				if(slimRunner.isSuccess()){
					FrontEnd.mainFrame.toolTab.statePanel.start(slimRunner.getBufferString(),true);
				}
				slimRunner = null;
				javax.swing.SwingUtilities.invokeLater(new Runnable(){public void run() {
					setButtonEnable(true);
				}});
			}})).start();

		}
	}

	public void runnerKill(){
		if(slimRunner!=null) slimRunner.kill();
	}

}
