package lmntaleditor.ltl;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.undo.UndoManager;

import lmntaleditor.Env;
import lmntaleditor.FrontEnd;
import lmntaleditor.Lang;
import lmntaleditor.runner.Ltl2baInstaller;
import lmntaleditor.runner.Ltl2baRunner;
import lmntaleditor.runner.SlimInstaller;
import lmntaleditor.runner.SlimRunner;
import lmntaleditor.util.CommonFontUser;
import lmntaleditor.util.SimpleUndoKeyListener;
import lmntaleditor.util.SimpleUndoableEditListener;

public class LtlNcPanel extends JPanel implements ActionListener,CommonFontUser {

	private JPanel panel;
	private JTextField ltlText;
	private UndoManager ltlUndoManager = new UndoManager();
	private JButton ltlButton;

	private JTextArea ncArea;

	LtlNcPanel(){

		setBorder(new TitledBorder("Never Claims"));
		setLayout(new BorderLayout());

		panel = new JPanel(new BorderLayout());
		panel.setBorder(new TitledBorder("LTL formula"));
		ltlText = new JTextField();
		ltlText.addActionListener(this);
		ltlText.getDocument().addUndoableEditListener(new SimpleUndoableEditListener(ltlUndoManager));
		ltlText.addKeyListener(new SimpleUndoKeyListener(ltlUndoManager));
		panel.add(ltlText, BorderLayout.CENTER);
		ltlButton = new JButton("Translate");
		ltlButton.addActionListener(this);
		panel.add(ltlButton, BorderLayout.EAST);
		add(panel,BorderLayout.NORTH);

		ncArea = new JTextArea();
		add(new JScrollPane(ncArea), BorderLayout.CENTER);

		loadFont();
		FrontEnd.addFontUser(this);

	}

	public void loadFont(){
		Font font = new Font(Env.get("EDITER_FONT_FAMILY"), Font.PLAIN, Env.getInt("EDITER_FONT_SIZE"));
		ncArea.setFont(font);
		ltlText.setFont(font);
		revalidate();
	}

	public void actionPerformed(ActionEvent e) {
		Object src = e.getSource();
		if(src==ltlButton||src==ltlText){

			final Ltl2baInstaller ltl2baInstaller = new Ltl2baInstaller();
			if(ltl2baInstaller.isNeedInstall()){
				if(ltl2baInstaller.isInstallable()){
					ltl2baInstaller.run();

					(new Thread(new Runnable() { public void run() {
						while(ltl2baInstaller.isRunning()){
							FrontEnd.sleep(200);
						}
						if(ltl2baInstaller.isSuccess()){
							ltlButton.doClick();
						}
					}})).start();
				}else{
					javax.swing.SwingUtilities.invokeLater(new Runnable(){public void run(){JOptionPane.showMessageDialog(
							FrontEnd.mainFrame,
							Lang.w[12]+" (lmntal/"+Env.LTL2BA_DIR_NAME+")",
							"LTL2BA INSTALL",
							JOptionPane.PLAIN_MESSAGE
					);}});
				}
				return;
			}

			final Ltl2baRunner runner = new Ltl2baRunner(ltlText.getText());
			runner.run();
			(new Thread(new Runnable() { public void run() {
				while(runner.isRunning()){
					FrontEnd.sleep(200);
				}
				javax.swing.SwingUtilities.invokeLater(new Runnable(){public void run() {
					ncArea.setText(runner.getOutput());
				}});
			}})).start();
		}
	}

	public void setText(String str){
		ncArea.setText(str);

		int l = str.indexOf("/*");
		int r = str.indexOf("*/");
		if(!(l>0&&r>0&&l<r)) return;

		String ltl = str.substring(l+2,r).trim();

		if(ltl.startsWith("!(")&&ltl.endsWith(")")){
			ltlText.setText(ltl.substring(2,ltl.length()-1).replaceAll("!!",""));
		}else{
			ltlText.setText(("!"+ltl).replaceAll("!!",""));
		}

		ltlUndoManager.discardAllEdits();

	}

	public String getText(){
		return ncArea.getText();
	}

}
