package lmntaleditor.mcprofiler;

import java.awt.BorderLayout;
import java.io.File;
import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;
import java.util.TreeMap;

import javax.swing.JPanel;

import lmntaleditor.runner.RunnerOutputGetter;

public class MCProfilePanel extends JPanel implements RunnerOutputGetter{

	private MCProfileGraphPanel graphPanel;
	private MCProfileLabelPanel label;
	private Timer clock;

	int hashConflict;
	ArrayList<State> allState;
	TreeMap<Long,ArrayList<State>> hashes;
	TreeMap<Integer,ArrayList<State>> suces;

	ArrayList<Integer[]> timeLine;

	public MCProfilePanel(){
		setLayout(new BorderLayout());

		allState = new ArrayList<State>();
		hashes = new TreeMap<Long,ArrayList<State>>();
		suces = new TreeMap<Integer,ArrayList<State>>();

		timeLine = new ArrayList<Integer[]>();

		graphPanel = new MCProfileGraphPanel(this);
		add(graphPanel,BorderLayout.CENTER);

		label = new MCProfileLabelPanel(this);
		add(label,BorderLayout.SOUTH);
	}

	void line(String str){
		String ss[] = str.split(":",4);
		if(ss.length<4){ return; }

		long id = Long.parseLong(ss[0]);
		long hash = Long.parseLong(ss[1]);
		int successor = Integer.parseInt(ss[2]);
		String s = ss[3];

		State state = new State(id,hash,successor,s);
		allState.add(state);

		//ϥå
		if(hashes.containsKey(hash)){
			hashConflict++;
			hashes.get(hash).add(state);
		}else{
			ArrayList<State> l = new ArrayList<State>();
			l.add(state);
			hashes.put(hash,l);
		}

		//
		if(suces.containsKey(successor)){
			suces.get(successor).add(state);
		}else{
			ArrayList<State> l = new ArrayList<State>();
			l.add(state);
			suces.put(successor,l);
		}
	}

	public void outputStart(String command, String option, File target) {
		hashConflict = 0;
		allState.clear();
		hashes.clear();
		suces.clear();

		timeLine.clear();
		timeLine.add(new Integer[]{0,0,0});

		graphPanel.start();
		label.update();
		repaint();

		clock = new Timer();
		clock.schedule(new Clock(), 1000, 1000);
	}

	public void outputLine(String str) {
		line(str);
	}

	public void outputEnd() {
		timeLine.add(new Integer[]{allState.size(),hashes.size(),hashConflict});
		clock.cancel();
		label.update();
		graphPanel.end();
		repaint();
	}

	class State{
		long id;
		long hash;
		int successor;
		String str;

		State(long id,long hash,int successor,String str){
			this.id = id;
			this.hash = hash;
			this.successor = successor;
			this.str = str;
		}
	}

	class Clock extends TimerTask{
		public void run(){
			timeLine.add(new Integer[]{allState.size(),hashes.size(),hashConflict});
			label.update();
			repaint();
		}
	}

}
