package lmntaleditor.runner;

import java.awt.Color;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.Timer;
import java.util.TimerTask;

import lmntaleditor.*;
import lmntaleditor.system.OutputPanel;

public class LmntalRunner {

	private ThreadRunner runner;
	private RunnerOutputGetter output;
	private StringBuffer buffer;
	
	private String option;
	private File targetFile;
	private boolean success;
	
	public LmntalRunner(String option){
		this(option,FrontEnd.mainFrame.editorPanel.getFile());
	}
	
	public LmntalRunner(String option,File targetFile){
		this.option = option;
		this.targetFile = targetFile;
		runner = new ThreadRunner();
		output = null;
		buffer = null;
		success = false;
	}

	public void run() {
		runner.start();
	}
	
	public void setOutputGetter(RunnerOutputGetter output){
		this.output = output;
	}
	
	public void setBuffering(boolean b){
		if(b){
			buffer = new StringBuffer();
		}else{
			buffer = null;
		}
	}
	
	public String getBufferString(){
		return buffer.toString();
	}

	public boolean isRunning() {
		if(runner==null) return false;
		return true;
	}

	public void kill() {
		if (runner!=null) {
			runner.kill();
			runner.interrupt();
			runner=null;
		}
	}
	
	public void exit(){
		runner=null;
	}
	
	public boolean isSuccess(){
		return success;
	}

	private class ThreadRunner extends Thread {
		
		private Process p;

		public void run() {
			try {
				String cmd = "java -DLMNTAL_HOME=lmntal -classpath lmntal"+File.separator+"bin"+File.separator+"lmntal.jar"+File.pathSeparator+"lmntal"+File.separator+"lib"+File.separator+"std_lib.jar runtime.FrontEnd --interpret "+option+" \""+targetFile.getAbsolutePath()+"\"";
				
				FrontEnd.println("(LMNtal) "+cmd);
				
				ProcessBuilder pb = new ProcessBuilder(strList(cmd));
				Env.setProcessEnvironment(pb.environment());
				pb.redirectErrorStream(true);
				p = pb.start();
				BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()));
				
				if(buffer==null){
					if(output==null){
						output = FrontEnd.mainFrame.toolTab.systemPanel.outputPanel;
					}
					output.outputStart("lmntal", option, targetFile);
					String str;
					while ((str=in.readLine())!=null) {
						output.outputLine(str);
					}
					output.outputEnd();
				}else{
					String str;
					while ((str=in.readLine())!=null) {
						buffer.append(str+"\n");
					}
				}
				
				in.close();
				p.waitFor();
				
				success = true;

			}catch(Exception e){
				FrontEnd.printException(e);
				
			}finally{
				exit();
			}
		}

		ArrayList<String> strList(String str){
			ArrayList<String> cmdList = new ArrayList<String>();
			StringTokenizer st = new StringTokenizer(str);
			while(st.hasMoreTokens()){
				String s = st.nextToken();
				if(s.length()>=2&&s.charAt(0)=='"'&&s.charAt(s.length()-1)=='"'){
					s = s.substring(1,s.length()-1);
				}
				cmdList.add(s);
			}
			return cmdList;
		}

		private void kill() {
			if(p!=null){
				p.destroy();
			}
		}

	}
	
}
