package lmntaleditor.runner;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Map;
import java.util.StringTokenizer;

import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;

import lmntaleditor.Env;
import lmntaleditor.FrontEnd;
import lmntaleditor.Lang;
import lmntaleditor.frame.ChildWindowListener;

public class Ltl2baInstaller {

	private ThreadRunner runner;
	private boolean success;

	public Ltl2baInstaller(){
		this.runner = null;
		this.success = false;
	}

	public boolean isNeedInstall(){
		File exe = new File("lmntal"+File.separator+Env.LTL2BA_DIR_NAME+File.separator+Env.getLtl2baBinaryName());
		return !exe.exists();
	}

	public boolean isInstallable(){
		File dir = new File("lmntal"+File.separator+Env.LTL2BA_DIR_NAME);
		return dir.exists();
	}

	public void run() {
		runner = new ThreadRunner();
		runner.start();
	}

	public boolean isRunning() {
		if(runner==null) return false;
		return true;
	}

	public void kill() {
		if (runner!=null) {
			runner.kill();
			runner.interrupt();
			runner=null;
		}
	}

	public void exit(){
		runner=null;
	}

	public boolean isSuccess(){
		return success;
	}

	private class ThreadRunner extends Thread {
		private Process p;
		private BufferedReader in;
		private InstallWindow window;

		ThreadRunner(){
			window = new InstallWindow();
			//new JDialog(window);
		}

		public void run() {
			try {

				ProcessBuilder pb;
				String str;
				String makeCmd = Env.getBinaryAbsolutePath("make");

				// makeư
				window.println(makeCmd);

				pb = new ProcessBuilder(strList(makeCmd));
				Env.setProcessEnvironment(pb.environment());
				pb.directory(new File("lmntal"+File.separator+Env.LTL2BA_DIR_NAME));
				pb.redirectErrorStream(true);
				p = pb.start();
				in = new BufferedReader(new InputStreamReader(p.getInputStream()));

				while ((str=in.readLine())!=null) {
					window.println(str);
				}
				in.close();
				p.waitFor();

				window.println("make end. exit="+p.exitValue()+".\n");

				javax.swing.SwingUtilities.invokeLater(new Runnable(){public void run(){JOptionPane.showMessageDialog(
						FrontEnd.mainFrame,
						Lang.w[10],
						"LTL2BA INSTALL",
						JOptionPane.PLAIN_MESSAGE
				);}});

				window.dispose();
				success = true;

			}catch(Exception e){

				javax.swing.SwingUtilities.invokeLater(new Runnable(){public void run(){JOptionPane.showMessageDialog(
						FrontEnd.mainFrame,
						Lang.w[11],
						"LTL2BA INSTALL",
						JOptionPane.PLAIN_MESSAGE
				);}});

				StringWriter sw = new StringWriter();
			    e.printStackTrace(new PrintWriter(sw));
			    window.println(sw.toString());
			    window.exit();

			}finally{

				exit();

			}
		}

		ArrayList<String> strList(String str){
			ArrayList<String> cmdList = new ArrayList<String>();
			StringTokenizer st = new StringTokenizer(str);
			while(st.hasMoreTokens()){
				String s = st.nextToken();
				if(s.length()>=2&&s.charAt(0)=='"'&&s.charAt(s.length()-1)=='"'){
					s = s.substring(1,s.length()-1);
				}
				cmdList.add(s);
			}
			return cmdList;
		}

		private void kill() {
			if(p!=null) p.destroy();
		}

	}

	private class InstallWindow extends JFrame implements ActionListener{

		private JProgressBar bar;
		private JTextArea text;
		private JButton button;

		InstallWindow(){

			setIconImage(Env.getImageOfFile("img/icon.gif"));
			setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
			setAlwaysOnTop(true);
			//setUndecorated(true);

		    JPanel panel = new JPanel();
		    panel.setBackground(new Color(255,255,255));
			panel.setLayout(new BoxLayout(panel,BoxLayout.Y_AXIS));
			add(panel);


			bar = new JProgressBar();
			bar.setIndeterminate(true);
			panel.add(bar);

			text = new JTextArea();
			text.setEditable(false);
			text.setLineWrap(true);
			text.setFont(new Font(null,Font.PLAIN,11));
			JScrollPane textScrollPane = new JScrollPane(text);
			textScrollPane.setPreferredSize(new Dimension(200,100));
			panel.add(textScrollPane);

			button = new JButton("OK");
			button.addActionListener(this);
			button.setEnabled(false);
			button.setAlignmentX(Component.CENTER_ALIGNMENT);
			panel.add(button);

			addWindowListener(new ChildWindowListener(this));

			pack();
			setLocationRelativeTo(null);
		    setVisible(true);

		}

		private void println(String str){
			text.append(str+"\n");
			text.setCaretPosition(text.getText().length()-1);
		}

		public void actionPerformed(ActionEvent e) {
			dispose();
		}

		public void exit(){
			javax.swing.SwingUtilities.invokeLater(new Runnable(){public void run(){
				bar.setIndeterminate(false);
				button.setEnabled(true);
			}});
		}

	}


}