package lmntaleditor.runner;

import java.awt.Color;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Timer;
import java.util.TimerTask;

import lmntaleditor.*;
import lmntaleditor.system.OutputPanel;

public class SlimRunner {

	private ThreadRunner runner;
	private RunnerOutputGetter output;
	private StringBuffer buffer;
	
	private String option;
	private File targetFile;
	private File symbolFile;
	private File ncFile;
	private boolean success;
	
	public SlimRunner(String option){
		this(option,FrontEnd.mainFrame.editorPanel.getFile());
	}
	
	public SlimRunner(String option,File targetFile){
		this.runner = new ThreadRunner();
		this.output = null;
		this.buffer = null;
		
		this.option = option;
		this.targetFile = targetFile;
		this.symbolFile = null;
		this.ncFile = null;
		this.success = false;
	}

	public void run() {
		runner.start();
	}
	
	public void setOutputGetter(RunnerOutputGetter output){
		this.output = output;
	}
	
	public void setBuffering(boolean b){
		if(b){
			buffer = new StringBuffer();
		}else{
			buffer = null;
		}
	}
	
	public void setSymbolFile(File symbolFile){
		this.symbolFile = symbolFile;
	}
	
	public void setNcFile(File ncFile){
		this.ncFile = ncFile;
	}
	
	public String getBufferString(){
		return buffer.toString();
	}

	public boolean isRunning() {
		if(runner==null) return false;
		return true;
	}

	public void kill() {
		if (runner!=null) {
			runner.kill();
			runner.interrupt();
			runner=null;
		}
	}
	
	public void exit(){
		runner=null;
	}
	
	public boolean isSuccess(){
		return success;
	}

	private class ThreadRunner extends Thread {
		private Process p1;
		private Process p2;
		private String slim_path;
		
		ThreadRunner(){
			slim_path = Env.get("SLIM_EXE_PATH");
			if(slim_path==null||slim_path.equals("")){ slim_path = "lmntal"+File.separator+"bin"+File.separator+Env.getSlimBinaryName(); }
		}
		
		public void run() {
			try {
				
				// LMNtalư
				String cmd1 = "java -DLMNTAL_HOME=lmntal -classpath lmntal"+File.separator+"bin"+File.separator+"lmntal.jar runtime.FrontEnd -O2 --interpret --slimcode \""+targetFile.getAbsolutePath()+"\"";
				
				FrontEnd.println("(SLIM) "+cmd1);
				
				ProcessBuilder pb = new ProcessBuilder(strList(cmd1));
				pb.directory(new File("."));
				Env.setProcessEnvironment(pb.environment());
				pb.redirectErrorStream(true);
				p1 = pb.start();
				BufferedInputStream in1 = new BufferedInputStream(p1.getInputStream());
				
				// SLIMư
				String cmd2 = slim_path+" "+option;
				if(symbolFile!=null){
					cmd2 += " --psym \""+symbolFile.getAbsolutePath()+"\" ";
					option += " --psym "+symbolFile.getName();
				}
				if(ncFile!=null){
					cmd2 += " --nc \""+ncFile.getAbsolutePath()+"\" ";
					option += " --nc "+ncFile.getName();
				}
				cmd2 += " -";
				FrontEnd.println("(SLIM) "+cmd2);
				
				pb = new ProcessBuilder(strList(cmd2));
				pb.directory(new File("."));
				Env.setProcessEnvironment(pb.environment());
				pb.redirectErrorStream(true);
				p2 = pb.start();
				OutputStreamWriter out2 = new OutputStreamWriter(p2.getOutputStream());
				BufferedReader in2 = new BufferedReader(new InputStreamReader(p2.getInputStream()));
				
				// SLIMή
				int b;
				while ((b = in1.read()) != -1) {
					out2.write(b);
				}
				try {
					out2.flush();
				}catch(Exception e){ FrontEnd.printException(e); } //ɸϤԤSLIMλ
				
				out2.close();
				in1.close();
				p1.waitFor();
				
				// SLIMνϤ
				if(buffer==null){
					if(output==null){
						output = FrontEnd.mainFrame.toolTab.systemPanel.outputPanel;
					}
					output.outputStart("slim", option, targetFile);
					String str;
					while ((str=in2.readLine())!=null) {
						output.outputLine(str);
					}
					output.outputEnd();
				}else{
					String str;
					while ((str=in2.readLine())!=null) {
						buffer.append(str+"\n");
					}
				}
				
				in2.close();
				p2.waitFor();
				
				success = true;
				
				/*
				InputStream in2 = p2.getInputStream();
				output = FrontEnd.mainFrame.toolTab.systemPanel.outputPanel;
				output.outputStart("slim", option, targetFile);
				int str = 0;
				//while ((str=in2.read())!=-1) {
				//	output.outputLine(String.valueOf(str));
				//}
				while(true){
					output.outputLine(String.valueOf(in2.available()));
					if(str++>100){ break; }
					Thread.sleep(250);
				}
				output.outputEnd();
				*/
				
				
				/*
				String cmd1 = "java -DLMNTAL_HOME=lmntal -classpath lmntal"+File.separator+"bin"+File.separator+"lmntal.jar runtime.FrontEnd -O2 --interpret --slimcode \""+targetFile.getAbsolutePath()+"\"";
				FrontEnd.println("(SLIM) "+cmd1);
				
				ProcessBuilder pb = new ProcessBuilder(strList(cmd1));
				pb.redirectErrorStream(true);
				p1 = pb.start();
				BufferedInputStream in1 = new BufferedInputStream(p1.getInputStream());
				
				FileOutputStream fos = new FileOutputStream(targetFile.getAbsolutePath()+".slimcode");
				int b = -1;
				while ((b = in1.read()) != -1) {
					fos.write(b);
				}
				fos.flush();
				fos.close();
				in1.close();
				p1.waitFor();
				
				String cmd2 = Env.get("SLIM_EXE_PATH")+" "+option+" \""+targetFile.getAbsolutePath()+".slimcode\"";
				FrontEnd.println("(SLIM) "+cmd2);
				
				pb = new ProcessBuilder(strList(cmd2));
				pb.redirectErrorStream(true);
				p2 = pb.start();
				
				BufferedReader in2 = new BufferedReader(new InputStreamReader(p2.getInputStream()));
				if(buffer==null){
					if(output==null){
						output = FrontEnd.mainFrame.toolTab.systemPanel.outputPanel;
					}
					output.outputStart("slim", option, targetFile);
					String str;
					while ((str=in2.readLine())!=null) {
						output.outputLine(str);
					}
					output.outputEnd();
				}else{
					String str;
					while ((str=in2.readLine())!=null) {
						buffer.append(str+"\n");
					}
				}
				
				in2.close();
				p2.waitFor();
				
				exit();
				success = true;
				
				*/

			}catch(Exception e){
				FrontEnd.printException(e);
				
			}finally{
				exit();
			}
			
		}
		
		ArrayList<String> strList(String str){
			ArrayList<String> cmdList = new ArrayList<String>();
			StringTokenizer st = new StringTokenizer(str);
			while(st.hasMoreTokens()){
				String s = st.nextToken();
				if(s.length()>=2&&s.charAt(0)=='"'&&s.charAt(s.length()-1)=='"'){
					s = s.substring(1,s.length()-1);
				}
				cmdList.add(s);
			}
			return cmdList;
		}

		private void kill() {
			if(p1!=null) p1.destroy();
			if(p2!=null) p2.destroy();
		}

	}
}
