package lmntaleditor.runner;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.StringTokenizer;

import lmntaleditor.Env;
import lmntaleditor.FrontEnd;

public class UnyoRunner {

	private ThreadRunner runner;
	
	private String option;
	private File targetFile;
	private boolean success;
	
	public UnyoRunner(String option){
		this(option,FrontEnd.mainFrame.editorPanel.getFile());
	}
	
	public UnyoRunner(String option,File targetFile){
		this.option = option;
		this.targetFile = targetFile;
		runner = new ThreadRunner();
		success = false;
	}

	public void run() {
		runner.start();
	}

	public boolean isRunning() {
		if(runner==null) return false;
		return true;
	}

	public void kill() {
		if (runner!=null) {
			runner.kill();
			runner.interrupt();
			runner=null;
		}
	}
	
	public void exit(){
		runner=null;
	}
	
	public boolean isSuccess(){
		return success;
	}

	private class ThreadRunner extends Thread {
		
		private Process p;

		public void run() {
			try {
				String cmd = "java -jar unyo.jar "+option+" \""+targetFile.getAbsolutePath()+"\"";
				
				FrontEnd.println("(UNYO) "+cmd);
				
				ProcessBuilder pb = new ProcessBuilder(strList(cmd));
				Env.setProcessEnvironment(pb.environment());
				pb.directory(new File("lmntal"+File.separator+Env.UNYO_DIR_NAME));
				pb.redirectErrorStream(true);
				p = pb.start();
				BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()));
				
				RunnerOutputGetter output = FrontEnd.mainFrame.toolTab.systemPanel.outputPanel;
				output.outputStart("java -jar unyo.jar", option, targetFile);
				String str;
				while ((str=in.readLine())!=null) {
					output.outputLine(str);
				}
				output.outputEnd();
				
				in.close();
				p.waitFor();
				
				success = true;

			}catch(Exception e){
				FrontEnd.printException(e);
				
			}finally{
				exit();
			}
		}

		ArrayList<String> strList(String str){
			ArrayList<String> cmdList = new ArrayList<String>();
			StringTokenizer st = new StringTokenizer(str);
			while(st.hasMoreTokens()){
				String s = st.nextToken();
				if(s.length()>=2&&s.charAt(0)=='"'&&s.charAt(s.length()-1)=='"'){
					s = s.substring(1,s.length()-1);
				}
				cmdList.add(s);
			}
			return cmdList;
		}

		private void kill() {
			if(p!=null){
				p.destroy();
			}
		}

	}
	
}
