package lmntaleditor.stateviewer;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingWorker;

import lmntaleditor.Env;
import lmntaleditor.FrontEnd;
import lmntaleditor.Lang;

public class StateGraphAdjustWorker extends SwingWorker<Object,Object>{
	private StateGraphPanel panel;
	private StatePainter painter;
	private StateNodeSet drawNodes;

	private ProgressFrame frame;

	public StateGraphAdjustWorker(StateGraphPanel panel,StatePainter painter,StateNodeSet drawNodes){
		this.panel = panel;
		this.painter = painter;
		this.drawNodes = drawNodes;
		painter.setActive(false);
		panel.repaint();

		frame = new ProgressFrame();
		addPropertyChangeListener(frame);
	}

	public void done() {
		panel.autoCentering();
		painter.update();
		painter.setActive(true);
		frame.dispose();
	}

	@Override
	protected Object doInBackground(){

		int endNode = 0;
		double w = (double)panel.getWidth();
		double h = (double)panel.getHeight();

		panel.setCenterZoom(1.0);

		double xInterval = 0;
		double yInterval = Double.MAX_VALUE;
		for(int i=0;i<drawNodes.getDepth();++i){
			double t = h/(drawNodes.getSameDepthSize(i)+1);
			if(t<yInterval){
				yInterval = t;
			}
		}
		if(yInterval>30&&(w/(drawNodes.getDepth()+1))>30){ xInterval=yInterval=30; }
		if(yInterval<10){ yInterval=10; }

		//Ŭx
		for(StateNode node : drawNodes.getAllNode()){
			node.setPosition((node.depth+1)*yInterval,node.getY());
		}

		//֤ܸʿ
		ArrayList<ArrayList<StateNode>> depthNode = drawNodes.getDepthNode();
		for(ArrayList<StateNode> nodes : depthNode){

			//ʿѾ
			for(StateNode node : nodes){
				int c = 0;
				double t = 0;
				for(StateNode from : node.getFromNodes()){
					if(from.depth<node.depth){
						++c;
						t += from.getY();
					}
				}
				if(c!=0){
					node.setPosition(node.getX(),t/c);
				}else{
					node.setPosition(node.getX(),0);
				}
			}

			//ۤ
			detanglingNodes(nodes,yInterval);

			//progress
			endNode += nodes.size();
			setProgress(100*endNode/(drawNodes.size()*3));
			if(isCancelled()){ return null; }

			/*
			//⤵¸
			double minY = Double.MAX_VALUE, maxY = Double.MIN_VALUE;
			for(StateNode node : nodes){
				if(node.getY()<minY){
					minY = node.getY();
				}
				if(maxY<node.getY()){
					maxY = node.getY();
				}
			}
			*/
		}
		/*
		//⤵ο
		int maxDepth = 0;
		double maxHeight = Double.MIN_VALUE;
		for(int i=0;i<depthHeight.size();++i){
			if(maxHeight<depthHeight.get(i)){
				maxDepth=i;
				maxHeight=depthHeight.get(i);
			}
		}

		//⤵ʿ֥Хåȥå
		for(int i=maxDepth-1;i>=0;--i){

			ArrayList<StateNode> nodes = depthNode.get(i);

			//ʿѾ
			for(StateNode node : nodes){
				int c = 0;
				double t = 0;
				for(StateNode to : node.getToNodes()){
					if(to.depth>node.depth){
						++c;
						t += to.getY();
					}
				}
				if(c!=0){
					node.setPosition(node.getX(),t/c);
				}
			}

			//ۤ(counterǤڤ)
			for(int counter=0;counter<100;++counter){
				boolean res = detanglingNodes(nodes,yInterval*3/2);
				if(!res) break;
			}
		}
		*/

		//ǿΡɤʿ
		for(int i=depthNode.size()-1;i>=0;--i){

			ArrayList<StateNode> nodes = depthNode.get(i);

			//ʿѾ
			for(StateNode node : nodes){
				int c = 0;
				double t = 0;
				for(StateNode to : node.getToNodes()){
					if(to.depth>node.depth){
						++c;
						t += to.getY();
					}
				}
				if(c!=0){
					node.setPosition(node.getX(),t/c);
				}
			}

			//ۤ
			detanglingNodes(nodes,yInterval);

			//progress
			endNode += nodes.size();
			setProgress(100*endNode/(drawNodes.size()*3));
			if(isCancelled()){ return null; }
		}


		//Ρɤ衦ܸʿ
		for(int i=0;i<depthNode.size();++i){

			ArrayList<StateNode> nodes = depthNode.get(i);

			//ʿѾ
			for(StateNode node : nodes){
				int c = 0;
				double t = 0;
				for(StateNode to : node.getToNodes()){
					if(to.depth-1==node.depth){
						++c;
						t += to.getY();
					}
				}
				for(StateNode from : node.getFromNodes()){
					if(from.depth+1==node.depth){
						++c;
						t += from.getY();
					}
				}
				if(c!=0){
					node.setPosition(node.getX(),t/c);
				}
			}

			//ۤ
			detanglingNodes(nodes,yInterval);

			//progress
			endNode += nodes.size();
			setProgress(100*endNode/(drawNodes.size()*3));
			if(isCancelled()){ return null; }
		}

		panel.autoCentering();

		if(xInterval==0){ xInterval = w/(drawNodes.getDepth()+1)/panel.getZoom(); }

		for(StateNode node : drawNodes.getAllNode()){
			node.setPosition((node.depth+1)*xInterval,node.getY());
		}

		return null;
	}

//	ۤ
	private void detanglingNodes(ArrayList<StateNode> nodes,double yInterval){

		//Ȥ
		ArrayList<StateNode> sortNodes = new ArrayList<StateNode>(nodes);
		Collections.sort(sortNodes, new Comparator<StateNode>() {
			public int compare(StateNode n1, StateNode n2) {
				if(n1.getY()<n2.getY()){
					return -1;
				}else if(n1.getY()>n2.getY()){
					return 1;
				}else{
					if(n1.no<n2.no){
						return -1;
					}else if(n1.no>n2.no){
						return 1;
					}else{
						return 0;
					}
				}
			}
		});

		ArrayList<StateNode> revsortNodes = new ArrayList<StateNode>(sortNodes);
		Collections.reverse(revsortNodes);

		//Υ0ΥΡɤۤ
		while(true){
			boolean res = zeroDetanlingNodes(sortNodes,revsortNodes,yInterval);
			if(!res||isCancelled()) break;
		}

		//ޤʴֳֶ
		double minY=Double.MAX_VALUE,maxY=Double.MIN_VALUE;
		for(StateNode node: nodes){
			minY = Math.min(minY,node.getY());
			maxY = Math.max(maxY,node.getY());
		}
		double par = (yInterval*nodes.size())/(maxY-minY);
		if((maxY-minY)>1&&par>1){
			double p = (maxY+minY)/2;
			for(StateNode node: nodes){
				node.setPosition(node.getX(),((node.getY()-p)*par)+p);
			}
		}

		//Ρдֳֶ
		for(int counter=0;counter<100;++counter){
			boolean res = nonzeroDetanlingNodes(sortNodes,revsortNodes,yInterval);
			if(!res||isCancelled()) break;
		}
	}

	private boolean zeroDetanlingNodes(ArrayList<StateNode> sortNodes,ArrayList<StateNode> revsortNodes,double yInterval){

		//Υ0ΥΡɤ򸫤Ĥ
		double zoreY = 0;
		HashSet<StateNode> zeroDistances = new HashSet<StateNode>();

		for(int i=0;i<sortNodes.size()-1;++i){
			StateNode n1 = sortNodes.get(i);
			StateNode n2 = sortNodes.get(i+1);
			double distance = Math.abs(n1.getY()-n2.getY());

			if(distance==0){
				if(zeroDistances.size()==0){
					zoreY = n2.getY();
					zeroDistances.add(n1);
					zeroDistances.add(n2);
				}else if(zoreY==n2.getY()){
					zeroDistances.add(n2);
				}
			}else{
				if(zeroDistances.size()>0){ break; }
			}
		}


		//Υ0ΥΡɤ̵ʤä齪λ
		if(zeroDistances.size()==0){ return false; }


		//ǾΥΣĤΥΡɤΰưΰ׻ν
		double dy;    //ư
		double pivot; //濴
		double moveMaxY; //ưΰ
		double moveMinY; //ưΰ貼

		for(StateNode n : sortNodes){ n.unmark(); }

		dy = yInterval*(zeroDistances.size()-1)/2;
		pivot = zoreY;

		moveMaxY = zoreY+dy+yInterval*3;
		moveMinY = zoreY-dy-yInterval*3;

		//noǥȤ
		List<StateNode> list = new ArrayList<StateNode>(zeroDistances);
		Collections.sort(list, new Comparator<StateNode>() {
			public int compare(StateNode n1, StateNode n2) {
				if(n1.no<n2.no){
					return -1;
				}else if(n1.no>n2.no){
					return 1;
				}else{
					return 0;
				}
			}
		});

		double y = pivot-dy;
		for(StateNode n : list){
			n.setPosition(n.getX(),y);
			n.mark();
			y += yInterval;
		}


		//ưΰ׻ʤΡɰư
		for(StateNode n : sortNodes){
			if(n.isMarked()) continue;
			if(n.getY()<pivot) continue;
			if(moveMaxY<n.getY()) break;
			moveMaxY += yInterval;
			n.setPosition(n.getX(), n.getY()+dy);
			n.mark();
		}
		for(StateNode n : revsortNodes){
			if(n.isMarked()) continue;
			if(n.getY()>pivot) continue;
			if(moveMinY>n.getY()) break;
			moveMinY -= yInterval;
			n.setPosition(n.getX(), n.getY()-dy);
			n.mark();
		}

		return true;
	}

	private boolean nonzeroDetanlingNodes(ArrayList<StateNode> sortNodes,ArrayList<StateNode> revsortNodes,double yInterval){

		//ǾΥΣĤΥΡɤ򸫤Ĥ
		StateNode move1=null,move2=null;
		double distanceMin = Double.MAX_VALUE;

		for(int i=0;i<sortNodes.size()-1;++i){
			StateNode n1 = sortNodes.get(i);
			StateNode n2 = sortNodes.get(i+1);
			double distance = Math.abs(n1.getY()-n2.getY());

			if(distance<distanceMin){
				distanceMin = distance;
				move1 = n1;
				move2 = n2;
			}
		}

		//ŤʤΤ̵ʤä齪λ
		if(distanceMin>yInterval){ return false; }

		//ǾΥΣĤΥΡɤΰưΰ׻ν
		double dy;    //ư
		double pivot; //濴
		double moveMaxY; //ưΰ
		double moveMinY; //ưΰ貼

		for(StateNode n : sortNodes){ n.unmark(); }

		dy = yInterval*(2-1)/2;
		pivot = (move1.getY()+move2.getY())/2;

		if(move1.getY()>=pivot){
			//move1>=move2
			moveMaxY = move1.getY()+dy+yInterval;
			moveMinY = move2.getY()-dy-yInterval;
			move1.setPosition(move1.getX(), move1.getY()+dy);
			move2.setPosition(move2.getX(), move2.getY()-dy);
			move1.mark();
			move2.mark();
		}else{
			//move1<move2
			moveMaxY = move2.getY()+dy+yInterval;
			moveMinY = move1.getY()-dy-yInterval;
			move2.setPosition(move2.getX(), move2.getY()+dy);
			move1.setPosition(move1.getX(), move1.getY()-dy);
			move2.mark();
			move1.mark();
		}


		//ưΰ׻ʤΡɰư
		for(StateNode n : sortNodes){
			if(n.isMarked()) continue;
			if(n.getY()<pivot) continue;
			if(moveMaxY<n.getY()) break;
			moveMaxY += yInterval;
			n.setPosition(n.getX(), n.getY()+dy);
			n.mark();
		}
		for(StateNode n : revsortNodes){
			if(n.isMarked()) continue;
			if(n.getY()>pivot) continue;
			if(moveMinY>n.getY()) break;
			moveMinY -= yInterval;
			n.setPosition(n.getX(), n.getY()-dy);
			n.mark();
		}

		return true;
	}


	private boolean innerDetanlingNodes(ArrayList<StateNode> sortNodes,ArrayList<StateNode> revsortNodes,double yInterval){

		//ǾΥΣĤΥΡɤ򸫤Ĥ
		StateNode move1=null,move2=null;
		double distanceMin = Double.MAX_VALUE;
		double zoreY = 0;
		HashSet<StateNode> zeroDistances = new HashSet<StateNode>();

		/*
		for(StateNode n1 : sortNodes){
			for(StateNode n2 : sortNodes){
				if(n1.no==n2.no){ break; }
				//if(n1.no<=n2.no){ continue; }
				double distance = Math.abs(n1.getY()-n2.getY());

				if(distance==0){
					distanceMin = 0;
					if(zeroDistances.size()==0){
						zoreY = n1.getY();
						zeroDistances.add(n1);
						zeroDistances.add(n2);
					}else if(zoreY==n1.getY()){
						zeroDistances.add(n1);
						zeroDistances.add(n2);
					}
				}else if(distance<distanceMin){
					distanceMin = distance;
					move1 = n1;
					move2 = n2;
				}
			}
		}
		*/

		for(int i=0;i<sortNodes.size()-1;++i){
			StateNode n1 = sortNodes.get(i);
			StateNode n2 = sortNodes.get(i+1);
			double distance = Math.abs(n1.getY()-n2.getY());
			if(zeroDistances.size()>0&&distance>0){ break; }

			if(distance==0){
				distanceMin = 0;
				if(zeroDistances.size()==0){
					zoreY = n2.getY();
					zeroDistances.add(n1);
					zeroDistances.add(n2);
				}else if(zoreY==n2.getY()){
					zeroDistances.add(n2);
				}
			}else if(distance<distanceMin){
				distanceMin = distance;
				move1 = n1;
				move2 = n2;
			}
		}

		//ŤʤΤ̵ʤä齪λ
		if(distanceMin>yInterval){ return false; }

		//ǾΥΣĤΥΡɤΰưΰ׻ν
		double dy;    //ư
		double pivot; //濴
		double moveMaxY; //ưΰ
		double moveMinY; //ưΰ貼

		for(StateNode n : sortNodes){ n.unmark(); }

		if(zeroDistances.size()==0){

			dy = yInterval*(2-1)/2;
			pivot = (move1.getY()+move2.getY())/2;

			if(move1.getY()>=pivot){
				//move1>=move2
				moveMaxY = move1.getY()+dy+yInterval;
				moveMinY = move2.getY()-dy-yInterval;
				move1.setPosition(move1.getX(), move1.getY()+dy);
				move2.setPosition(move2.getX(), move2.getY()-dy);
				move1.mark();
				move2.mark();
			}else{
				//move1<move2
				moveMaxY = move2.getY()+dy+yInterval;
				moveMinY = move1.getY()-dy-yInterval;
				move2.setPosition(move2.getX(), move2.getY()+dy);
				move1.setPosition(move1.getX(), move1.getY()-dy);
				move2.mark();
				move1.mark();
			}

		}else{

			dy = yInterval*(zeroDistances.size()-1)/2;
			pivot = zoreY;

			moveMaxY = zoreY+dy+yInterval*3;
			moveMinY = zoreY-dy-yInterval*3;

			//noǥȤ
			List<StateNode> list = new ArrayList<StateNode>(zeroDistances);
			Collections.sort(list, new Comparator<StateNode>() {
				public int compare(StateNode n1, StateNode n2) {
					if(n1.no<n2.no){
						return -1;
					}else if(n1.no>n2.no){
						return 1;
					}else{
						return 0;
					}
				}
			});

			double y = pivot-dy;
			for(StateNode n : list){
				n.setPosition(n.getX(),y);
				n.mark();
				y += yInterval;
			}
		}

		//ưΰ׻ʤΡɰư
		for(StateNode n : sortNodes){
			if(n.isMarked()) continue;
			if(n.getY()<pivot) continue;
			if(moveMaxY<n.getY()) break;
			moveMaxY += yInterval;
			n.setPosition(n.getX(), n.getY()+dy);
			n.mark();
		}
		for(StateNode n : revsortNodes){
			if(n.isMarked()) continue;
			if(n.getY()>pivot) continue;
			if(moveMinY>n.getY()) break;
			moveMinY -= yInterval;
			n.setPosition(n.getX(), n.getY()-dy);
			n.mark();
		}
		/*
		boolean hit=true;
		while(hit){
			hit = false;
			for(StateNode n : nodes){
				if(n.isMarked()) continue;
				if(pivot<=n.getY()&&n.getY()<=moveMaxY){
					moveMaxY += yInterval;
					n.setPosition(n.getX(), n.getY()+dy);
					n.mark();
					//nowMaxY = Math.max(nowMaxY,n.getY());
					hit = true;
				}else if(moveMinY<=n.getY()&&n.getY()<pivot){
					moveMinY -= yInterval;
					n.setPosition(n.getX(), n.getY()-dy);
					n.mark();
					//nowMinY = Math.min(nowMinY,n.getY());
					hit = true;
				}
			}
		}
		*/

		return true;
	}



	/*
	private ArrayList<StateNode> getHitNodes(double yInterval,ArrayList<StateNode> noes, ArrayList<StateNode> hits){
		ArrayList<StateNode> newhits = new ArrayList<StateNode>();
		for(StateNode n : noes){
			for(StateNode h : hits){
				if(Math.abs(n.getY()-h.getY())<=yInterval){
					newhits.add(n);
				}
			}
		}
		//ҥåȤΡɤСƵŪˤ˿ҥåȤΡɤõ
		if(newhits.size()>0){
			noes.removeAll(newhits);
			hits.addAll(newhits);
			newhits.addAll(getHitNodes(yInterval,noes,newhits));
		}
		return newhits;
	}

	private StateNode getMinYNode(ArrayList<StateNode> nodes){
		StateNode min = null;
		double minY= Double.MAX_VALUE;
		for(StateNode node : nodes){
			if(node.getY()<minY){
				minY = node.getY();
				min = node;
			}
		}
		return min;
	}
	*/


	private class ProgressFrame extends JDialog implements PropertyChangeListener,ActionListener {
		private JPanel panel;
		private JProgressBar bar;
		private JButton cancel;

		private ProgressFrame(){
			panel = new JPanel();

			bar = new JProgressBar(0,100);
			bar.setStringPainted(true);
			panel.add(bar);

			cancel = new JButton(Lang.d[2]);
			cancel.addActionListener(this);
			panel.add(cancel);

			add(panel);

			setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
			setTitle("Adjust Reset");
			setIconImage(Env.getImageOfFile("img/icon.gif"));
			setAlwaysOnTop(true);
			setResizable(false);

	        pack();
	        setLocationRelativeTo(panel);
	        setVisible(true);
		}

		public void propertyChange(PropertyChangeEvent evt) {
			if ("progress".equals(evt.getPropertyName())) {
				bar.setValue((Integer)evt.getNewValue());
			}
		}

		public void actionPerformed(ActionEvent e) {
			Object src = e.getSource();
			if(src==cancel){
				if(!isDone()){
					cancel(true);
				}
			}
		}
	}


}
