package lmntaleditor.stateviewer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RectangularShape;
import java.awt.geom.RoundRectangle2D;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;

import javax.imageio.ImageIO;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.event.MouseInputListener;

import lmntaleditor.*;
import lmntaleditor.runner.LmntalRunner;
import lmntaleditor.stateviewer.StateNode.ColorMode;
import lmntaleditor.util.CommonFontUser;

public class StateGraphPanel extends JPanel implements MouseInputListener,MouseWheelListener,KeyListener,CommonFontUser{

	public StatePanel statePanel;

	private StateNodeSet drawNodes;

	public ArrayList<StateNode> selectNodes = new ArrayList<StateNode>();
	private boolean nodeSelected;

	private RightMenu rightMenu = new RightMenu(this);

	private double zoom;
	private boolean simpleMode;

	private Point lastPoint;
	private StatePainter painter;
	private double xPosInterval;
	private double[] yPosInterval;

	private JPanel status = new JPanel();
	private JLabel zoomNum = new JLabel();
	private JLabel stateNum = new JLabel();

	private JTextField clickStringField;

	public StateGraphPanel(StatePanel statePanel){
		this.statePanel = statePanel;

		setFocusable(true);
		addMouseListener(this);
		addMouseMotionListener(this);
		addMouseWheelListener(this);
		addKeyListener(this);

		setLayout(new BorderLayout());

		status.setLayout(new GridLayout(1,2));
		status.setOpaque(false);
		setZoomNum(1.0);
		zoomNum.setOpaque(false);
		zoomNum.setHorizontalAlignment(JLabel.LEFT);
		status.add(zoomNum);
		setStateNum(0,0);
		stateNum.setOpaque(false);
		stateNum.setHorizontalAlignment(JLabel.RIGHT);
		status.add(stateNum);
		add(status, BorderLayout.NORTH);

		clickStringField = new JTextField();
		clickStringField.setVisible(false);
		add(clickStringField, BorderLayout.SOUTH);

		painter = new StatePainter(this);
		painter.start();

		loadFont();
		FrontEnd.addFontUser(this);
	}

	public void init(StateNodeSet nodes){

		drawNodes = nodes;
		selectNodes.clear();
		nodeSelected = false;
		zoom = 1.0;
		lastPoint = null;
		xPosInterval = 20;
		yPosInterval = null;

		positionReset();

		painter.setActive(true);

		statePanel.stateControlPanel.allButtonSetEnabled(true);

	}

	public void loadFont(){
		Font font = new Font(Env.get("EDITER_FONT_FAMILY"), Font.PLAIN, Env.getInt("EDITER_FONT_SIZE"));
		clickStringField.setFont(font);
		revalidate();
		//Dimension d = clickStringField.getPreferredSize();
		//clickStringField.setLocation(0,this.getHeight()-d.height);
		//clickStringField.setSize(getWidth(),d.height);
	}

	private StateNode pickANode(Point p){
		StateNode pick = null;
		for(StateNode node : drawNodes.getAllNodeDrawOrder()){
			if(node == null){ continue; }
			Rectangle rect = node.getBounds();
			if(rect.contains(p)){
				pick = node;
			}
		}
		return pick;
	}

	public void setZoom(double zoom){
		if(zoom<0.01){ zoom=0.01; }else if(zoom>4.0){ zoom=4.0; }
		this.zoom = zoom;
		statePanel.stateControlPanel.setSliderPos(zoom);
	}

	public void setCenterZoom(double zoom){
		if(zoom<0.01){ zoom=0.01; }else if(zoom>4.0){ zoom=4.0; }

		double w = (double)getWidth() / 2;
		double h = (double)getHeight() / 2;
		double newW = w / zoom;
		double newH = h / zoom;
		double oldW = w / this.zoom;
		double oldH = h / this.zoom;
		allMove(newW-oldW,newH-oldH);
		this.zoom = zoom;
		statePanel.stateControlPanel.setSliderPos(zoom);
	}

	public void setSliderZoom(double zoom){
		if(zoom<0.01){ zoom=0.01; }else if(zoom>4.0){ zoom=4.0; }

		double w = (double)getWidth() / 2;
		double h = (double)getHeight() / 2;
		double newW = w / zoom;
		double newH = h / zoom;
		double oldW = w / this.zoom;
		double oldH = h / this.zoom;
		allMove(newW-oldW,newH-oldH);
		this.zoom = zoom;

		painter.update();
	}

	public double getZoom(){
		return zoom;
	}

	void allMove(double dx, double dy){
		for(StateNode node : drawNodes.getAllNode()){
			node.move(dx, dy);
		}
	}

	void positionReset(){
		double w = (double)getWidth();
		double h = (double)getHeight();

		setCenterZoom(1.0);
		xPosInterval = w/(drawNodes.getDepth()+1);
		yPosInterval = new double[drawNodes.getDepth()];
		for(int i=0;i<drawNodes.getDepth();++i){
			yPosInterval[i] = h/(drawNodes.getSameDepthSize(i)+1);
		}

		for(StateNode node : drawNodes.getAllNode()){
			node.resetLocation(xPosInterval,yPosInterval);
		}
		painter.update();
	}

	void lengthReset(){
		double w = (double)getWidth();
		double h = (double)getHeight();

		double minLength = w/(drawNodes.getDepth()+1);
		for(int i=0;i<drawNodes.getDepth();++i){
			double d = h/(drawNodes.getSameDepthSize(i)+1);
			if(d<minLength) minLength = d;
		}
		setCenterZoom(minLength/30);
		xPosInterval = w/(drawNodes.getDepth()+1);
		xPosInterval /= getZoom();
		yPosInterval = new double[drawNodes.getDepth()];
		for(int i=0;i<drawNodes.getDepth();++i){
			yPosInterval[i] = h/(drawNodes.getSameDepthSize(i)+1);
			yPosInterval[i] /= getZoom();
		}

		for(StateNode node : drawNodes.getAllNode()){
			node.resetLocation(xPosInterval,yPosInterval);
		}
		painter.update();
	}

	void adjustReset(){

		StateGraphAdjustWorker worker = new StateGraphAdjustWorker(this,painter,drawNodes);
		worker.execute();

	}

	void groupMove(){
		for(int i=0;i<5;++i){
			for(StateNode node : drawNodes.getAllNode()){
				double y=0;
				int count=0;
				for(StateNode to : node.getToNodes()){
					y+=to.getY();
					++count;
				}
				for(StateNode from : node.getFromNodes()){
					y+=from.getY();
					++count;
				}
				node.setPosition(node.getX(),y/count);
			}
		}
		painter.update();
	}

	public void autoCentering(){
		Rectangle2D.Double d = getNodesDimension();

		allMove(-1.0*d.x,-1.0*d.y);

		double w = (double)getWidth();
		double h = (double)getHeight();
		double zoomX = w/d.width;
		double zoomY = h/d.height;

		if(zoomX<zoomY){
			setZoom(zoomX);
			allMove(0,((h-d.height*getZoom())/2.0)/getZoom());
		}else{
			setZoom(zoomY);
			allMove(((w-d.width*getZoom())/2.0)/getZoom(),0);
		}
		painter.update();
	}

	public Rectangle2D.Double getNodesDimension(){
		double minX=Double.MAX_VALUE,minY=Double.MAX_VALUE,maxX=Double.MIN_VALUE,maxY=Double.MIN_VALUE;

		for(StateNode node : drawNodes.getAllNode()){
			if(node.getX()<minX){ minX = node.getX(); }else if(maxX<node.getX()){ maxX = node.getX(); }
			if(node.getY()<minY){ minY = node.getY(); }else if(maxY<node.getY()){ maxY = node.getY(); }
		}

		minX -= 24.0;
		minY -= 14.0;
		maxX += 14.0;
		maxY += 14.0;

		return new Rectangle2D.Double(minX,minY,maxX-minX,maxY-minY);
	}

	public int stateFind(String str){

		int match = 0;
		selectNodes.clear();
		nodeSelected = false;
		drawNodes.allNodeColorMode(StateNode.ColorMode.weak);

		if(str.equals("")){
			drawNodes.allNodeColorMode(StateNode.ColorMode.normal);
			painter.update();
			return 0;
		}

		for(StateNode node : drawNodes.getAllNode()){
			if(node.isMatch(str)){
				drawNodes.setOrderEnd(node);
				node.setColorMode(StateNode.ColorMode.normal);
				node.updateLooks();
				match++;
			}
		}
		painter.update();
		return match;
	}

	public int stateMatch(String head,String guard){

		int match = 0;
		selectNodes.clear();
		nodeSelected = false;
		drawNodes.allNodeColorMode(StateNode.ColorMode.weak);

		if(head.equals("")&&guard.equals("")){
			drawNodes.allNodeColorMode(StateNode.ColorMode.normal);
			painter.update();
			return 0;
		}

		File f = new File("temp.lmn");
		try {
			FileWriter fp = new FileWriter(f);
			fp.write(drawNodes.getMatchFileString(head,guard));
            fp.close();
		} catch (IOException e) {
			FrontEnd.printException(e);
		}

		final LmntalRunner lr = new LmntalRunner("",f);
		lr.setBuffering(true);
		lr.run();

		while(lr.isRunning()){
			FrontEnd.sleep(200);
		}
		if(lr.isSuccess()){
			String str = lr.getBufferString();
			int sp=str.indexOf("sVr_matches{");
			if(sp==-1){ return -1; }
			int ep=str.indexOf("}",sp+12);
			String ids = str.substring(sp+12,ep);
			if(ids.length()==0){
				match = 0;
			}else{
				String[] strs = ids.split(",");
				for(String s: strs){
					int id = Integer.parseInt(s.substring(s.indexOf("(")+1,s.indexOf(")")));
					StateNode node = drawNodes.get(id);
					drawNodes.setOrderEnd(node);
					node.setColorMode(StateNode.ColorMode.normal);
					node.updateLooks();
					match++;
				}
			}
		}else{
			drawNodes.allNodeColorMode(StateNode.ColorMode.normal);
			painter.update();
			return -1;
		}

		painter.update();
		return match;
	}

	private void setStateString(String str){
		if(str==null){
			clickStringField.setVisible(false);
		}else{
			clickStringField.setVisible(true);
			clickStringField.setText(str);
			validate();
		}
	}

	public void allDelete(){
		painter.setActive(false);
		setStateNum(0,0);
		repaint();
	}

	private void setZoomNum(double zoom){
		zoomNum.setText(" Zoom : "+(int)(zoom*100)+"% ");
	}

	private void setStateNum(int num,int end){
		stateNum.setText("State : "+num+" (End : "+end+") ");
	}

	public void paintComponent(Graphics g){
		Graphics2D g2 = (Graphics2D)g;

		g2.setColor(Color.white);
		g2.fillRect(0, 0, getWidth(), getHeight());

		if(!painter.isActive()){ return; }

		setZoomNum(getZoom());
		setStateNum(drawNodes.size(),drawNodes.getEndNode().size());

		g2.scale(zoom,zoom);

		drawGraph(g2,getWidth(),getHeight(),zoom<=0.7);

		double reZoom = 1.0/zoom;
		g2.scale(reZoom, reZoom);
	}

	public void drawGraph(Graphics2D g2,int width,int height,boolean simpleMode){
		this.simpleMode = simpleMode;

		//оݤη
		double minX=-10/zoom,maxX=(width+10)/zoom;
		double minY=-10/zoom,maxY=(height+10)/zoom;
		for(StateNode node : drawNodes.getAllNodeDrawOrder()){
			node.unmark();
			if(minX<node.getX()&&node.getX()<maxX&&minY<node.getY()&&node.getY()<maxY){
				node.mark();
			}
		}

		//֤
		drawStartArrow(g2);

		//
		for(StateNode node : drawNodes.getAllNodeDrawOrder()){
			drawArrow(g2,node);
		}

		//Ρɤ
		for(StateNode node : drawNodes.getAllNodeDrawOrder()){
			drawNode(g2,node);
		}

		//򤷤ƤΡɤ
		for(StateNode node : selectNodes){
			drawSelectNodeAndLink(g2,node);
		}
		if(selectNodes.size()==1){
			setStateString(selectNodes.get(0).state);
		}else{
			setStateString(null);
		}
	}

	private void drawStartArrow(Graphics2D g2){
		StateNode node = drawNodes.getStartNode();
		if(node==null){ return; }
		if(!node.isMarked()){ return; }

		Shape shape = node.getShape();
		Rectangle rect = shape.getBounds();
		int radius = (int)node.getRadius();
		if(node.getColorMode()==ColorMode.weak){
			g2.setColor(Color.lightGray);
		}else{
			g2.setColor(Color.black);
		}

		g2.drawLine(rect.x-10-6,rect.y+radius,rect.x-6,rect.y+radius);
		g2.drawLine(rect.x-3-6,rect.y+radius-2,rect.x-6,rect.y+radius);
		g2.drawLine(rect.x-3-6,rect.y+radius+2,rect.x-6,rect.y+radius);
	}

	private void drawNode(Graphics2D g2,StateNode node){
		if(!node.isMarked()){ return; }
		Shape shape = node.getShape();
		g2.setColor(node.getFillColor());
		g2.fill(shape);
		if(!simpleMode){
			g2.setColor(node.getDrawColor());
			g2.draw(shape);
			if(node.isAccept()){
				double r = node.getRadius()-2.0;
				Shape doubleShape = new RoundRectangle2D.Double(node.getX()-r,node.getY()-r,r*2,r*2,r*2,r*2);
				g2.draw(doubleShape);
			}
		}
	}

	private void drawArrow(Graphics2D g2,StateNode node){
		Shape shape = node.getShape();
		Rectangle rect = shape.getBounds();
		int radius = (int)node.getRadius();

		//ؤɽ
		if(!simpleMode){
			for(StateNode to : node.getToNodes()){
				if(node.getColorMode()==ColorMode.weak||to.getColorMode()==ColorMode.weak||!drawNodes.isFlow(node,to)){
					g2.setColor(Color.lightGray);
				}else{
					g2.setColor(Color.black);
				}
				if(to==node){
					if(!node.isMarked()){ continue; }
					g2.drawArc(rect.x-radius,rect.y-radius,radius*2,radius*2,0,270);
					g2.drawLine(rect.x-2,rect.y+radius-3,rect.x,rect.y+radius);
					g2.drawLine(rect.x-3,rect.y+radius+1,rect.x,rect.y+radius);
				}else{
					if(!node.isMarked()&&!to.isMarked()){ continue; }
					Rectangle c_rect = to.getShape().getBounds();
					int c_radius = (int)to.getRadius();
					drawNodeArrow(g2,rect.x+radius,rect.y+radius,c_rect.x+c_radius,c_rect.y+c_radius,5,c_radius);
				}
			}
		}else{
			for(StateNode to : node.getToNodes()){
				if(node.getColorMode()==ColorMode.weak||to.getColorMode()==ColorMode.weak||!drawNodes.isFlow(node,to)){
					g2.setColor(Color.lightGray);
				}else{
					g2.setColor(Color.black);
				}
				if(to!=node){
					if(!node.isMarked()&&!to.isMarked()){ continue; }
					Rectangle c_rect = to.getShape().getBounds();
					int c_radius = (int)to.getRadius();
					g2.drawLine(rect.x+radius,rect.y+radius,c_rect.x+c_radius,c_rect.y+c_radius);
				}
			}
		}
	}

	private void drawSelectNodeAndLink(Graphics2D g2,StateNode node){
		Shape shape = node.getShape();
		Rectangle rect = shape.getBounds();
		int radius = (int)node.getRadius();

		// ܸɽ
		g2.setColor(Color.BLUE);
		for(StateNode from : node.getFromNodes()){
			Rectangle c_rect = from.getShape().getBounds();
			int c_radius = (int)from.getRadius();
			if(!simpleMode){
				drawNodeArrow(g2,c_rect.x+c_radius,c_rect.y+c_radius,rect.x+radius,rect.y+radius,5,c_radius);
			}else{
				g2.drawLine(rect.x+radius,rect.y+radius,c_rect.x+c_radius,c_rect.y+c_radius);
			}
		}

		// ɽ
		g2.setColor(Color.RED);
		for(StateNode to : node.getToNodes()){
			if(to==node){
				g2.drawArc(rect.x-radius,rect.y-radius,radius*2,radius*2,0,270);
				g2.drawLine(rect.x-2,rect.y+radius-3,rect.x,rect.y+radius);
				g2.drawLine(rect.x-3,rect.y+radius+1,rect.x,rect.y+radius);
			}else{
				Rectangle c_rect = to.getShape().getBounds();
				int c_radius = (int)to.getRadius();
				if(!simpleMode){
					drawNodeArrow(g2,rect.x+radius,rect.y+radius,c_rect.x+c_radius,c_rect.y+c_radius,5,c_radius);
				}else{
					g2.drawLine(rect.x+radius,rect.y+radius,c_rect.x+c_radius,c_rect.y+c_radius);
				}
				//뤫å
				boolean flag = false;
				for(StateNode t : to.getToNodes()){
					if(t==node){ flag=true;break; }
				}
				if(flag){
					if(!simpleMode){
						drawNodeArrow(g2,c_rect.x+c_radius,c_rect.y+c_radius,rect.x+radius,rect.y+radius,5,radius);
					}else{
						g2.drawLine(rect.x+radius,rect.y+radius,c_rect.x+c_radius,c_rect.y+c_radius);
					}
				}
			}

		}

		g2.setColor(node.getFillColor());
		g2.fill(shape);
		g2.setColor(Color.RED);
		g2.draw(shape);
	}

	private double getMinLength(long id,double x,double y){
		double minLength2 = 100000000;
		for(StateNode node : drawNodes.getAllNode()){
			if(node.id!=id){
				double dx = x-node.getX();
				double dy = y-node.getY();
				double l = dx*dx+dy*dy;
				if(l<minLength2){
					minLength2 = l;
				}
			}
		}
		return Math.sqrt(minLength2);
	}


	private double getSumLength(int id,double x,double y){
		double sumLength = 0;
		for(StateNode node : drawNodes.getAllNode()){
			if(node.id!=id){
				double dx = x-node.getX();
				double dy = y-node.getY();
				sumLength += dx*dx+dy*dy;
			}
		}
		return sumLength;
	}

	private double getLinkLength(StateNode node,double x,double y){
		double sumLength = 0;
		for(StateNode to : node.getToNodes()){
			double dx = x-to.getX();
			double dy = y-to.getY();
			sumLength += dx*dx+dy*dy;
		}
		for(StateNode from : node.getFromNodes()){
			double dx = x-from.getX();
			double dy = y-from.getY();
			sumLength += dx*dx+dy*dy;
		}
		return sumLength;
	}

	void reduction(){
		drawNodes.reduction();
		painter.update();
	}

	public void shakeMove(){
		for(int i=0;i<50;++i){
			for(StateNode node : drawNodes.getAllNode()){
				double newX = (Math.random()-0.5)*20.0 + node.getX();
				double newY = (Math.random()-0.5)*20.0 + node.getY();
				double nowMinLength = getMinLength(node.id,node.getX(),node.getY());
				double newMinLength = getMinLength(node.id,newX,newY);
				if(nowMinLength<24&&nowMinLength<newMinLength){
					node.setPosition(newX, newY);
				}else if(newMinLength>=24){
					double nowLinkLength = getLinkLength(node,node.getX(),node.getY());
					double newLinkLength = getLinkLength(node,newX,newY);
					if(newLinkLength<nowLinkLength){
						node.setPosition(newX, newY);
					}
				}
			}
		}
		autoCentering();
	}

	private void drawNodeArrow(Graphics2D g2,int x0,int y0,int x1,int y1,int l,int r){
	    int x,y;
	    double theta = Math.atan2((double)(y1-y0),(double)(x1-x0));
	    int dx = (int)((r+1)*Math.cos(theta));
	   	int dy = (int)((r+1)*Math.sin(theta));

	    g2.drawLine(x0+dx,y0+dy,x1-dx,y1-dy);
	   	double dts = Math.PI / 6.0;
	   	x = x1-(int)(l*Math.cos(theta-dts));
	    y = y1-(int)(l*Math.sin(theta-dts));
	    g2.drawLine(x1-dx,y1-dy,x-dx,y-dy);
	    x = x1-(int)(l*Math.cos(theta+dts));
	    y = y1-(int)(l*Math.sin(theta+dts));
	    g2.drawLine(x1-dx,y1-dy,x-dx,y-dy);
	}

	public void mouseClicked(MouseEvent e) {

	}

	public void mouseEntered(MouseEvent e) {

	}

	public void mouseExited(MouseEvent e) {

	}

	public void mousePressed(MouseEvent e) {
		if(!painter.isActive()){ return; }
		requestFocus();

		lastPoint = e.getPoint();
		Point p = new Point((int)((double)e.getX()/zoom), (int)((double)e.getY()/zoom));
		StateNode selectNode = pickANode(p);
		if(e.isControlDown()){
			if(selectNode!=null){
				if(!selectNodes.contains(selectNode)){
					selectNodes.add(selectNode);
					nodeSelected = true;
				}else{
					selectNodes.remove(selectNode);
					nodeSelected = false;
				}
			}else{
				nodeSelected = false;
			}
		}else{
			if(selectNode!=null){
				if(!selectNodes.contains(selectNode)){
					selectNodes.clear();
					selectNodes.add(selectNode);
				}else if(e.getClickCount()==2){
					selectNode.showFrame(e.getXOnScreen(), e.getYOnScreen());
				}
				nodeSelected = true;
			}else{
				selectNodes.clear();
				nodeSelected = false;
			}
		}
		if(SwingUtilities.isRightMouseButton(e)){
			rightMenu.updateEnabled();
			rightMenu.show(e.getComponent(), e.getX(), e.getY());
		}
		painter.update();
	}

	public void mouseReleased(MouseEvent e) {
		if(!painter.isActive()){ return; }
		lastPoint = null;
	}

	public void mouseDragged(MouseEvent e) {
		if(!painter.isActive()){ return; }
		if(lastPoint == null){
			lastPoint = e.getPoint();
			return;
		}
		double dx = (double)(e.getX() - lastPoint.x) / zoom;
		double dy = (double)(e.getY() - lastPoint.y) / zoom;
		lastPoint = e.getPoint();

		if(nodeSelected){
			for(StateNode node : selectNodes){
				if(e.isShiftDown()){
					node.move(dx, dy);
				}else{
					accompany_move(node,dx,dy);
				}
			}
		}else{
			allMove(dx, dy);
		}
		painter.update();
	}

	private void accompany_move(StateNode node,double dx,double dy){
		drawNodes.allNodeUnMark();

		LinkedList<StateNode> now_queue = new LinkedList<StateNode>();
		LinkedList<StateNode> next_queue = new LinkedList<StateNode>();

		next_queue.add(node);

		int count=0;
		while(!next_queue.isEmpty()){
			now_queue.clear();
			for(StateNode n : next_queue){
				now_queue.add(n);
			}
			next_queue.clear();

			for(StateNode n : now_queue){
				if(n.isMarked()){ continue; }
				n.mark();
				n.move(dx, dy);

				for(StateNode to : n.getToNodes()){
					next_queue.add(to);
				}
				for(StateNode from : n.getFromNodes()){
					next_queue.add(from);
				}
			}
			dx /= 2;
			dy /= 2;
			count++;
			if(count>10){ break; }
		}
	}

	public void mouseMoved(MouseEvent e) {
		// TODO ư줿᥽åɡ

	}

	public void mouseWheelMoved(MouseWheelEvent e) {
		if(!painter.isActive()){ return; }
		double z = Math.sqrt(getZoom()*100);
		z -= (double)e.getWheelRotation()/2.0;
		setCenterZoom(z*z/100.0);
		painter.update();
	}


	public void keyPressed(KeyEvent e) {
		if(!painter.isActive()){ return; }

		switch(e.getKeyCode()){
		case KeyEvent.VK_LEFT:
			for(StateNode node : selectNodes){
				node.move(-5/this.getZoom(),0);
			}
			break;
		case KeyEvent.VK_RIGHT:
			for(StateNode node : selectNodes){
				node.move(5/this.getZoom(),0);
			}
			break;
		case KeyEvent.VK_DOWN:
			for(StateNode node : selectNodes){
				node.move(0,5/this.getZoom());
			}
			break;
		case KeyEvent.VK_UP:
			for(StateNode node : selectNodes){
				node.move(0,-5/this.getZoom());
			}
			break;
		case KeyEvent.VK_DELETE:
			for(StateNode node : selectNodes){
				drawNodes.remove(node);
			}
			selectNodes.clear();
			painter.update();
			break;
		}
		painter.update();
	}

	public void keyReleased(KeyEvent e) {

	}

	public void keyTyped(KeyEvent e) {

	}


	private class RightMenu extends JPopupMenu implements ActionListener{
		private StateGraphPanel panel;

		private JMenuItem remove = new JMenuItem("Remove");
		private JMenuItem unyo = new JMenuItem("Unyo");
		private JMenuItem add = new JMenuItem("add Editor");

		private JMenuItem posReset = new JMenuItem("Position Reset");
		private JMenuItem lengthReset = new JMenuItem("Length Reset");
		private JMenuItem adjustReset = new JMenuItem("Adjust Reset");

		private JMenuItem autoCentering = new JMenuItem("Auto Centering");
		private JMenuItem shakeMove = new JMenuItem("Shake Move");
		private JMenuItem groupMove = new JMenuItem("Group Move");

		private JCheckBoxMenuItem match = new JCheckBoxMenuItem("Match State");
		private JCheckBoxMenuItem find = new JCheckBoxMenuItem("Find State");

		private JMenuItem reduction = new JMenuItem("Reduction");
		//private JMenuItem output = new JMenuItem("Output");
		private JMenuItem dotFile = new JMenuItem("Save Dot File");

		private JMenuItem allReset = new JMenuItem("All Reset");
		private JMenuItem allDelete = new JMenuItem("All Delete");


		RightMenu(StateGraphPanel panel){
			this.panel = panel;

			remove.addActionListener(this);
			add(remove);

			unyo.addActionListener(this);
			add(unyo);

			add.addActionListener(this);
			add(add);

			addSeparator();


			posReset.addActionListener(this);
			add(posReset);

			lengthReset.addActionListener(this);
			add(lengthReset);

			adjustReset.addActionListener(this);
			add(adjustReset);

			addSeparator();


			autoCentering.addActionListener(this);
			add(autoCentering);

			shakeMove.addActionListener(this);
			add(shakeMove);

			groupMove.addActionListener(this);
			add(groupMove);

			addSeparator();


			match.addActionListener(this);
			match.setSelected(Env.is("SV_MATCH"));
			add(match);

			find.addActionListener(this);
			find.setSelected(Env.is("SV_FIND"));
			add(find);

			addSeparator();


			reduction.addActionListener(this);
			add(reduction);

			//output.addActionListener(this);
			//add(output);

			dotFile.addActionListener(this);
			add(dotFile);

			addSeparator();


			allReset.addActionListener(this);
			add(allReset);

			allDelete.addActionListener(this);
			add(allDelete);
		}

		public void actionPerformed(ActionEvent e) {
			JMenuItem src = (JMenuItem)e.getSource();
			if(src==remove){
				for(StateNode node : selectNodes){
					drawNodes.remove(node);
				}
				selectNodes.clear();
				painter.update();
			}else if(src==unyo){
				for(StateNode node : selectNodes){
					node.runUnyo();
				}
			}else if(src==add){
				for(StateNode node : selectNodes){
					FrontEnd.mainFrame.editorPanel.editor.replaceSelection(node.state);
				}
			}else if(src==posReset){
				positionReset();
			}else if(src==lengthReset){
				lengthReset();
			}else if(src==adjustReset){
				adjustReset();
			}else if(src==autoCentering){
				autoCentering();
			}else if(src==shakeMove){
				shakeMove();
			}else if(src==groupMove){
				groupMove();
			}else if(src==find){
				statePanel.stateControlPanel.toggleFindVisible();
			}else if(src==match){
				statePanel.stateControlPanel.toggleMatchVisible();
			}else if(src==reduction){
				reduction();
				/*
			}else if(src==output){
				int w = (int)(panel.getWidth()/panel.getZoom());
				int h = (int)(panel.getHeight()/panel.getZoom());
				BufferedImage bi = new BufferedImage(w,h,BufferedImage.TYPE_INT_RGB);
				Graphics2D g2 = (Graphics2D)bi.getGraphics();
				g2.setColor(Color.white);
				g2.fillRect(0, 0, w, h);
				drawGraph(g2,w,h,false);
				try {
				    ImageIO.write(bi, "jpg", new File("output.jpg"));
				} catch (Exception ex) {
				    FrontEnd.printException(ex);
				}
				*/
			}else if(src==dotFile){
				dotfileSave();
				FrontEnd.mainFrame.toolTab.setTab("System");
			}else if(src==allReset){
				statePanel.reset();
			}else if(src==allDelete){
				statePanel.stateControlPanel.allButtonSetEnabled(false);
				allDelete();
			}
		}

		void updateEnabled(){
			remove.setEnabled(true);
			unyo.setEnabled(true);
			add.setEnabled(true);

			if(selectNodes.size()==0){
				remove.setEnabled(false);
			}
			if(selectNodes.size()!=1){
				unyo.setEnabled(false);
				add.setEnabled(false);
			}

		}

		private boolean dotfileSave(){

			File lmn = FrontEnd.mainFrame.editorPanel.getFile();
			File file = new File(lmn.getAbsolutePath()+".dot");
			if (file!=null) {
				try {
					BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file.getAbsolutePath())));
					writer.write("digraph sample {\n");
					writer.write("graph [rankdir = LR];\n");
					writer.write("node [style=filled,color=\"#336666\"];\n");
					writer.write(statePanel.stateGraphPanel.drawNodes.getDotString());
					//writer.write(statePanel.stateGraphPanel.drawNodes.getRankString());
					writer.write("}\n");
					writer.close();
	            	FrontEnd.println("(StateViewer) dot file save. [ "+file.getName()+" ]");
	            	FrontEnd.println("(StateViewer) example:\ndot -Tgif \""+file.getAbsolutePath()+"\" -o \""+(new File(lmn.getAbsolutePath()+".gif")).getAbsolutePath()+"\"");
	            	return true;
				} catch (IOException e) {
					FrontEnd.printException(e);
				}
			}

			return false;
		}


	}


}
