package lmntaleditor.stateviewer;

import java.awt.Color;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.geom.RectangularShape;
import java.awt.geom.RoundRectangle2D;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JTextArea;

import lmntaleditor.Env;
import lmntaleditor.FrontEnd;
import lmntaleditor.frame.ChildWindowListener;
import lmntaleditor.runner.LmntalRunner;
import lmntaleditor.runner.UnyoRunner;

class StateNode{
	public long id;
	public int no;
	public int depth = 0;
	public int nth = 0;
	public String state;
	boolean accept;

	//ǽˤȤʤ,toNodes,fromNodesꤵnull
	public ArrayList<Long> toIds;
	public ArrayList<Long> fromIds;

	//ꤵưʹߤϤѤ
	private ArrayList<StateNode> toNodes;
	private ArrayList<StateNode> fromNodes;

	private boolean marked;
	private Shape shape;
	private ColorMode colorMode;
	private double radius;
	private Color fillColor;
	private Color drawColor;
	enum ColorMode { normal, em, weak };

	StateNode(long id,int no,String toIdsStr,String state){
		this.id = id;
		this.no = no;
		this.depth = 0;
		this.nth = 0;
		this.state = state;
		this.accept = false;

		this.toIds = new ArrayList<Long>();
		if(toIdsStr.length()>0){
			String[] toIdsStrs = toIdsStr.split(",");
			for(String toId : toIdsStrs){
				toIds.add(Long.parseLong(toId));
			}
		}
		this.fromIds = new ArrayList<Long>();

		this.marked = false;
		this.shape = new RoundRectangle2D.Double(0,0,0,0,0,0);
		this.colorMode = ColorMode.normal;
	}

	void updateLooks(){

		double to = toNodes.size();
		double from = fromNodes.size();

		double r = 0;
		double g = 0;
		double b = 0;

		if(to<from){
			if(to*2<from){
				r = 255;
				g = 255*Math.sqrt(to*2/from);
			}else{
				g = 255;
				r = 255*Math.sqrt(to/from);
			}
		}else if(to>from){
			if(to>from*2){
				b = 255;
				g = 255*Math.sqrt(from*2/to);
			}else{
				g = 255;
				b = 255*Math.sqrt(from/to);
			}
		}else{
			g = 255;
		}

		switch(colorMode){
			case normal:
				radius = 5.0;
				fillColor = new Color((int)r,(int)g,(int)b);
				drawColor = Color.gray;
				break;
			case em:
				radius = 6.0;
				fillColor = new Color((int)r,(int)g,(int)b);
				drawColor = Color.black;
				break;
			case weak:
				radius = 4.0;
				fillColor = Color.white;
				drawColor = new Color((int)r,(int)g,(int)b);
				break;
		}

		shape = new RoundRectangle2D.Double(getX(),getY(),radius*2,radius*2,radius*2,radius*2);

		/*
		if(toIds.size()==0){
			color = Color.red;
		}else if(toIds.size()==1){
			color = Color.blue;
		}else if(toIds.size()==2){
			color = Color.orange;
		}else if(toIds.size()==3){
			color = Color.green;
		}else if(toIds.size()==4){
			color = Color.yellow;
		}else{
			color = Color.cyan;
		}
		*/
	}

	double getX(){
		return ((RectangularShape)getShape()).getCenterX();
	}

	double getY(){
		return ((RectangularShape)getShape()).getCenterY();
	}

	void move(double dx, double dy){
		setPosition(getX() + dx, getY() + dy);
	}

	void setPosition(double x,double y){
		((RectangularShape)getShape()).setFrame(x-radius, y-radius, radius*2, radius*2);
	}

	void initToAndFromNode(ArrayList<StateNode> toNodes,ArrayList<StateNode> fromNodes){
		this.toNodes = toNodes;
		this.fromNodes = fromNodes;
		toIds = null;
		fromIds = null;
	}

	void addToNode(StateNode toNode){
		if(!toNodes.contains(toNode)){
			toNodes.add(toNode);
		}
	}
	void removeToNode(StateNode toNode){
		toNodes.remove(toNode);
	}
	void addFromNode(StateNode fromNode){
		if(!fromNodes.contains(fromNode)){
			fromNodes.add(fromNode);
		}
	}
	void removeFromNode(StateNode fromNode){
		fromNodes.remove(fromNode);
	}

	ArrayList<StateNode> getToNodes(){
		return toNodes;
	}

	ArrayList<StateNode> getFromNodes(){
		return fromNodes;
	}


	Shape getShape(){
		return shape;
	}

	double getRadius(){
		return radius;
	}

	Rectangle getBounds(){
		return shape.getBounds();
	}

	Color getDrawColor(){
		return drawColor;
	}

	Color getFillColor(){
		return fillColor;
	}

	ColorMode getColorMode(){
		return colorMode;
	}

	void setColorMode(ColorMode colorMode){
		this.colorMode = colorMode;
	}

	boolean isAccept(){
		return accept;
	}

	void setAccept(boolean accept){
		this.accept = accept;
	}

	void mark(){
		marked = true;
	}

	void unmark(){
		marked = false;
	}

	boolean isMarked(){
		return marked;
	}

	void resetLocation(double xPosInterval,double[] yPosInterval){
		double x = (depth+1)*xPosInterval;
		double y = (nth+1)*yPosInterval[depth];
		setPosition(x,y);
	}

	String getStringTo(int no){
		String fillcolor = Integer.toHexString((getFillColor().getRGB())&0xffffff);
		while(fillcolor.length()<6){ fillcolor = "0"+fillcolor; }
		String toes = id+" [fillcolor=\"#"+fillcolor+"\",label=\""+no+"\"];\n";
		for(StateNode to : toNodes){
			toes += id+" -> "+to.id+";\n";
		}
		return toes;
	}

	void runUnyo(){
		File f = new File("temp.lmn");
		try {
			FileWriter fp = new FileWriter(f);
			fp.write(state);
            fp.close();
		} catch (IOException e) {
			FrontEnd.printException(e);
		}
		//final LmntalRunner lr = new LmntalRunner("-g",f);
		//lr.run();
		(new UnyoRunner(Env.get("LMNTAL_OPTION"),f)).run();
	}

	boolean isMatch(String str){
		if(state.lastIndexOf(str)>=0){
			return true;
		}else{
			return false;
		}
	}

	void showFrame(int x, int y){
		TextFrame frame = new TextFrame(""+id,state);
		frame.setBounds(x,y,300,100);
	}

	private class TextFrame extends JFrame{

		TextFrame(String title,String state){

			setTitle(title);
			setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

			JTextArea text = new JTextArea(state);
			text.setLineWrap(true);
			text.setFont(new Font(Env.get("EDITER_FONT_FAMILY"), Font.PLAIN, Env.getInt("EDITER_FONT_SIZE")));

		    add(text);

		    addWindowListener(new ChildWindowListener(this));

		    setVisible(true);

		}
	}

	/*
	private class TextDialog extends JDialog{

		TextDialog(String title,String state){

			super(FrontEnd.mainFrame);

			setTitle(title);

			JTextArea text = new JTextArea(state);
			text.setLineWrap(true);
			text.setFont(new Font(Env.get("EDITER_FONT_FAMILY"), Font.PLAIN, Env.getInt("EDITER_FONT_SIZE")));

		    add(text);

		    setVisible(true);

		}
	}
	*/

	/*
	boolean isLmntalMatch(String head,String guard){
		File f = new File("temp.lmn");
		try {
			FileWriter fp = new FileWriter(f);
			fp.write("sVr_nomatch,"+state+"\n");
			fp.write("sVr_nomatch,"+head+" :- ");
			if(!guard.equals("")){ fp.write(guard+" | "); }
			fp.write(head+".");
            fp.close();
		} catch (IOException e) {
			FrontEnd.printException(e);
		}

		final LmntalRunner lr = new LmntalRunner("",f);
		lr.setBuffering(true);
		lr.run();

		while(lr.isRunning()){
			FrontEnd.sleep(200);
		}
		if(lr.isSuccess()){
			String str = lr.getBufferString();
			if(str.indexOf("sVr_nomatch")==-1){
				return true;
			}
		}
		return false;
	}
	*/

}
